from __future__ import annotations

from typing import (
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import recycle_bin

class _RecycleBin(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/RecycleBin')
        self.DeletionTask = self._DeletionTask(api_obj=self._api_obj, url=f'{self._url}/DeletionTask')
        self.Empty = self._Empty(api_obj=self._api_obj, url=f'{self._url}/Empty')
        self.GetConfiguration = self._GetConfiguration(api_obj=self._api_obj, url=f'{self._url}/GetConfiguration')
        self.GetContents = self._GetContents(api_obj=self._api_obj, url=f'{self._url}/GetContents')
        self.GetItem = self._GetItem(api_obj=self._api_obj, url=f'{self._url}/GetItem')
        self.GetItemDetails = self._GetItemDetails(api_obj=api_obj, url=f'{self._url}/GetItemDetails')
        self.Purge = self._Purge(api_obj=self._api_obj, url=f'{self._url}/Purge')
        self.PurgeTask = self._PurgeTask(api_obj=self._api_obj, url=f'{self._url}/PurgeTask')
        self.Restore = self._Restore(api_obj=self._api_obj, url=f'{self._url}/Restore')
        self.SetConfiguration = self._SetConfiguration(api_obj=self._api_obj, url=f'{self._url}/SetConfiguration')

    class _DeletionTask(WebSdkEndpoint):
        def post(self, start: bool = None, stop: bool = None):
            body = {
                'Start': start,
                'Stop' : stop
            }

            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Empty(WebSdkEndpoint):
        def post(self):
            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _GetConfiguration(WebSdkEndpoint):
        def post(self):
            class Output(WebSdkOutputModel):
                deletion: recycle_bin.Deletion = ApiField(alias='Deletion')
                purge: recycle_bin.Purge = ApiField(alias='Purge')
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _GetContents(WebSdkEndpoint):
        def post(self, limit: int):
            body = {
                'Limit': limit
            }

            class Output(WebSdkOutputModel):
                items: list[recycle_bin.Item] = ApiField(alias='Items', default_factory=list)
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))
                total: int = ApiField(alias='Total')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetItem(WebSdkEndpoint):
        def post(self, guid: str):
            body = {
                'Guid': guid
            }

            class Output(WebSdkOutputModel):
                item: recycle_bin.Item = ApiField(alias='Item')
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetItemDetails(WebSdkEndpoint):
        def post(self, guid: str):
            body = {
                'Guid': guid
            }

            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))
                details: list[recycle_bin.Detail] = ApiField(alias='Details')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Purge(WebSdkEndpoint):
        def post(self, guid: str):
            body = {
                'Guid': guid
            }

            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _PurgeTask(WebSdkEndpoint):
        def post(self, purge_all: bool, start: bool = None, stop: bool = None):
            body = {
                'PurgeAll': purge_all,
                'Start'   : start,
                'Stop'    : stop
            }

            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Restore(WebSdkEndpoint):
        def post(self, guid: str):
            body = {
                'Guid': guid
            }

            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _SetConfiguration(WebSdkEndpoint):
        def post(self, deletion: Union[dict, recycle_bin.Deletion], purge: Union[dict, recycle_bin.Purge] = None):
            body = {
                'Deletion': deletion,
                'Purge'   : purge
            }

            class Output(WebSdkOutputModel):
                result: recycle_bin.Result = ApiField(alias='Result', converter=lambda x: recycle_bin.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.service_module import ServiceModuleAttributes

class CAImportManagerAttributes(ServiceModuleAttributes, metaclass=IterableMeta):
    __config_class__ = "CA Import Manager"



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from datetime import datetime
from typing import (
    Any,
    Dict,
    List,
    Literal,
)
from uuid import UUID

AnyValue = Any

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class TagAssignmentErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    message: str = ApiField(alias='message')
    statusCode: int = ApiField(alias='statusCode')

class TagInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    id: UUID = ApiField(alias='id')
    isReserved: bool = ApiField(alias='isReserved')
    key: str = ApiField(alias='key')
    name: str = ApiField(alias='name')

class TagRequest(ObjectModel):
    name: str = ApiField(alias='name')
    values: List[str] = ApiField(alias='values', default_factory=list)

class TagResponse(ObjectModel):
    count: int = ApiField(alias='count')
    tags: List[TagInformation] = ApiField(alias='tags', default_factory=list)

class TagValueInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    id: UUID = ApiField(alias='id')
    tagId: UUID = ApiField(alias='tagId')
    value: str = ApiField(alias='value')

class TagValuesRequest(ObjectModel):
    values: List[str] = ApiField(alias='values', default_factory=list)

class TagValuesResponse(ObjectModel):
    count: int = ApiField(alias='count')
    values: List[TagValueInformation] = ApiField(alias='values', default_factory=list)

class TagsAndValuesInformation(ObjectModel):
    tag: TagInformation = ApiField(alias='tag')
    values: List[TagValueInformation] = ApiField(alias='values', default_factory=list)

class TagsAndValuesResponse(ObjectModel):
    tagsAndValues: List[TagsAndValuesInformation] = ApiField(alias='tagsAndValues', default_factory=list)

class TagsAssignRequest(ObjectModel):
    action: Literal['ADD', 'DELETE', 'DELETE_ALL', 'REPLACE'] = ApiField(alias='action')
    entityIds: List[UUID] = ApiField(alias='entityIds', default_factory=list)
    entityType: Literal['APPLICATION', 'CERTIFICATE', 'CERTIFICATE_INSTANCE'] = ApiField(alias='entityType')
    targetedTags: List[str] = ApiField(alias='targetedTags', default_factory=list)

class TagsAssignResponse(ObjectModel):
    tagsAssignInformation: List[TagsAssignResponseInformation] = ApiField(
        alias='tagsAssignInformation',
        default_factory=list
    )

class TagsAssignResponseInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    entityId: UUID = ApiField(alias='entityId')
    entityType: str = ApiField(alias='entityType')
    errorInformation: TagAssignmentErrorInformation = ApiField(alias='errorInformation')
    modificationDate: datetime = ApiField(alias='modificationDate')
    status: str = ApiField(alias='status')
    tags: List[str] = ApiField(alias='tags', default_factory=list)

class TagsAssignmentAggregatesRequest(ObjectModel):
    entityType: Literal['APPLICATION', 'CERTIFICATE', 'CERTIFICATE_INSTANCE'] = ApiField(alias='entityType')
    tags: List[str] = ApiField(alias='tags', default_factory=list)

class TagsAssignmentAggregatesResponse(ObjectModel):
    aggregates: Dict[str, List[TagsAssignmentAggregatesResponseInformation]] = ApiField(
        alias='aggregates',
        default_factory=dict
    )

class TagsAssignmentAggregatesResponseInformation(ObjectModel):
    count: int = ApiField(alias='count')
    entityType: str = ApiField(alias='entityType')

class TagsBulkRequest(ObjectModel):
    tags: List[TagRequest] = ApiField(alias='tags', default_factory=list)

ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
TagAssignmentErrorInformation.update_forward_refs()
TagInformation.update_forward_refs()
TagRequest.update_forward_refs()
TagResponse.update_forward_refs()
TagValueInformation.update_forward_refs()
TagValuesRequest.update_forward_refs()
TagValuesResponse.update_forward_refs()
TagsAndValuesInformation.update_forward_refs()
TagsAndValuesResponse.update_forward_refs()
TagsAssignRequest.update_forward_refs()
TagsAssignResponse.update_forward_refs()
TagsAssignResponseInformation.update_forward_refs()
TagsAssignmentAggregatesRequest.update_forward_refs()
TagsAssignmentAggregatesResponse.update_forward_refs()
TagsAssignmentAggregatesResponseInformation.update_forward_refs()
TagsBulkRequest.update_forward_refs()



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.credential_base import CredentialBaseAttributes

class CertificateCredentialAttributes(CredentialBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Certificate Credential"
    certificate = Attribute('Certificate')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class ImpervaMXAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Imperva MX"
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    key_store_validation_disabled = Attribute('Key Store Validation Disabled', min_version='21.4')
    private_key_name = Attribute('Private Key Name', min_version='21.4')
    server_group = Attribute('Server Group', min_version='21.4')
    service = Attribute('Service', min_version='21.4')
    site = Attribute('Site', min_version='21.4')
    username_credential = Attribute('Username Credential', min_version='21.4')
    utility_path = Attribute('Utility Path', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class LogApplicationCustomizationAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Log Application Customization"
    customization_data = Attribute('Customization Data', min_version='21.4')
    language = Attribute('Language', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.service_module import ServiceModuleAttributes

class MonitoringModuleAttributes(ServiceModuleAttributes, metaclass=IterableMeta):
    __config_class__ = "Monitoring Module"
    configuration = Attribute('Configuration', min_version='23.3')
    escalation_notice_interval = Attribute('Escalation Notice Interval', min_version='21.4')
    escalation_notice_start = Attribute('Escalation Notice Start', min_version='21.4')
    expiration_notice_interval = Attribute('Expiration Notice Interval', min_version='21.4')
    expiration_notice_start = Attribute('Expiration Notice Start', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.identity_driver import IdentityDriverAttributes

class LDAPIdentityDriverAttributes(IdentityDriverAttributes, metaclass=IterableMeta):
    __config_class__ = "LDAP Identity Driver"
    additional_name_attributes = Attribute('Additional Name Attributes', min_version='22.3')
    ambiguous_name_resolution = Attribute('Ambiguous Name Resolution', min_version='21.4')
    base_dn = Attribute('Base DN', min_version='21.4')
    configuration = Attribute('Configuration', min_version='21.4')
    connection_timeout = Attribute('Connection Timeout', min_version='21.4')
    container_class_name = Attribute('Container Class Name', min_version='21.4')
    container_query_expression = Attribute('Container Query Expression', min_version='21.4')
    dsn = Attribute('DSN', min_version='21.4')
    friendly_name = Attribute('Friendly Name', min_version='21.4')
    group_class_name = Attribute('Group Class Name', min_version='21.4')
    group_query_expression = Attribute('Group Query Expression', min_version='21.4')
    group_search_root = Attribute('Group Search Root', min_version='21.4')
    host = Attribute('Host', min_version='21.4')
    internal_identifier = Attribute('Internal Identifier', min_version='21.4')
    lockout_timestamp_attribute = Attribute('Lockout Timestamp Attribute', min_version='22.3')
    login_name_resolution = Attribute('Login Name Resolution', min_version='21.4')
    member_identifier = Attribute('Member Identifier', min_version='21.4')
    memberof_enabled = Attribute('MemberOf Enabled', min_version='21.4')
    membership_resolution = Attribute('Membership Resolution', min_version='21.4')
    port = Attribute('Port', min_version='21.4')
    resolve_nested_groups = Attribute('Resolve Nested Groups', min_version='21.4')
    revocation = Attribute('Revocation', min_version='21.4')
    search_root = Attribute('Search Root', min_version='21.4')
    secure = Attribute('Secure', min_version='21.4')
    size_limit = Attribute('Size Limit', min_version='21.4')
    timeout = Attribute('Timeout', min_version='21.4')
    title = Attribute('Title', min_version='21.4')
    universal_identifier = Attribute('Universal Identifier', min_version='21.4')
    user_class_name = Attribute('User Class Name', min_version='21.4')
    user_query_expression = Attribute('User Query Expression', min_version='21.4')
    vault_id = Attribute('Vault Id', min_version='21.4')
    vendor = Attribute('Vendor', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.credential_driver_base import CredentialDriverBaseAttributes

class AdaptableCredentialConnectorAttributes(CredentialDriverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Adaptable Credential Connector"
    credential = Attribute('Credential', min_version='21.4')
    powershell_script = Attribute('PowerShell Script', min_version='21.4')
    powershell_script_hash_vault_id = Attribute('PowerShell Script Hash Vault Id', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class CAPIAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "CAPI"
    binding_ip_address = Attribute('Binding IP Address', min_version='21.4')
    binding_port = Attribute('Binding Port', min_version='21.4')
    create_binding = Attribute('Create Binding', min_version='21.4')
    crypto_service_provider = Attribute('Crypto Service Provider', min_version='21.4')
    delete_previous_cert_and_key = Attribute('Delete Previous Cert And Key', min_version='23.3')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    friendly_name = Attribute('Friendly Name', min_version='21.4')
    hostname = Attribute('Hostname', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    non_exportable = Attribute('Non-Exportable', min_version='21.4')
    private_key_label = Attribute('Private Key Label', min_version='21.4')
    private_key_location = Attribute('Private Key Location', min_version='21.4')
    private_key_trustee = Attribute('Private Key Trustee', min_version='21.4')
    store_name = Attribute('Store Name', min_version='23.3')
    update_iis = Attribute('Update IIS', min_version='21.4')
    web_site_name = Attribute('Web Site Name', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class FlowAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Flow"
    archive_process = Attribute('Archive Process')
    archive_validity = Attribute('Archive Validity')
    product_code = Attribute('Product Code')
    timeout = Attribute('Timeout')



from __future__ import annotations

from typing import (
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import hsm_api

class _HSMAPI(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/API')
        self._url = self._url.replace('vedsdk', 'vedhsm')
        self.Derive = self._Derive(api_obj=api_obj, url=f'{self._url}/derive')
        self.GetChain = self._GetChain(api_obj=api_obj, url=f'{self._url}/getchain')
        self.GetGPGPublicKey = self._GetGPGPublicKey(api_obj=self._api_obj, url=f'{self._url}/GetGPGPublicKey')
        self.GetObjects = self._GetObjects(api_obj=self._api_obj, url=f'{self._url}/GetObjects')
        self.IsHsm = self._IsHsm(api_obj=api_obj, url=f'{self._url}/IsHsm')
        self.Sign = self._Sign(api_obj=self._api_obj, url=f'{self._url}/Sign')
        self.SignJWT = self._SignJWT(api_obj=self._api_obj, url=f'{self._url}/SignJWT')
        self.StoreObject = self._StoreObject(api_obj=api_obj, url=f'{self._url}/storeobject')

    class _IsHsm(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                vedauth: str = ApiField(alias='vedauth')
                pks: str = ApiField(alias='pks')
                timestamp: str = ApiField(alias='timestamp')
                csc: str = ApiField(alias='csc')

            return generate_output(output_cls=Output, response=self._get())

    class _Derive(WebSdkEndpoint):
        def post(self):
            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                error: str = ApiField(alias='Error')
                try_later: bool = ApiField(alias='TryLater')
                request_thumbprint: str = ApiField(alias='RequestThumbprint')
                result_data: str = ApiField(alias='ResultData')

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _StoreObject(WebSdkEndpoint):
        def post(
            self,
            key_id: str = None,
            key_creation_timestamp: str = None,
            key_context: str = None,
            public_key: str = None,
            private_key: str = None,
            private_key_password: str = None,
        ):
            body = {
                "KeyId"               : key_id,
                "KeyCreationTimestamp": key_creation_timestamp,
                "KeyContext"          : key_context,
                "PublicKey"           : public_key,
                "PrivateKey"          : private_key,
                "PrivateKeyPassword"  : private_key_password,
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetChain(WebSdkEndpoint):
        def post(self, key_id: str):
            body = {
                "KeyId": key_id,
            }

            class Output(WebSdkOutputModel):
                certificates: list[hsm_api.Certificate] = ApiField(alias='Certificates')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Sign(WebSdkEndpoint):
        def post(
            self,
            client_info: Union[dict, hsm_api.ClientInfo],
            data: str,
            key_context: str,
            key_id: int,
            mechanism: int,
            process_info: Union[dict, hsm_api.ProcessInfo],
            client_mechanism: str = None,
            justification: str = None,
            key_context_to_wrap: int = None,
            key_id_to_wrap: int = None,
            parameter: Union[dict, hsm_api.Parameter] = None,
            password: str = None,
            username: str = None,
            verify_data: bool = None,
            wrapping_key_id: int = None
        ):
            body = {
                'ClientInfo'      : client_info,
                'ClientMechanism' : client_mechanism,
                'Data'            : data,
                'Justification'   : justification,
                'KeyContext'      : key_context,
                'KeyContextToWrap': key_context_to_wrap,
                'KeyId'           : key_id,
                'KeyIdToWrap'     : key_id_to_wrap,
                'Mechanism'       : mechanism,
                'Parameter'       : parameter,
                'Password'        : password,
                'ProcessInfo'     : process_info,
                'Username'        : username,
                'VerifyData'      : verify_data,
                'WrappingKeyId'   : wrapping_key_id
            }

            class Output(WebSdkOutputModel):
                result_data: str = ApiField(alias='ResultData')
                success: bool = ApiField(alias='Success')
                try_later: bool = ApiField(alias='TryLater')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _SignJWT(WebSdkEndpoint):
        def post(
            self,
            client_info: Union[dict, hsm_api.ClientInfo],
            process_info: Union[dict, hsm_api.ProcessInfo],
            key_id: str,
            header: str,
            payload: str
        ):
            body = {
                'ClientInfo' : client_info,
                'ProcessInfo': process_info,
                'KeyId'      : key_id,
                'Header'     : header,
                'Payload'    : payload
            }

            class Output(WebSdkOutputModel):
                result_data: str = ApiField(alias='ResultData')
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetGPGPublicKey(WebSdkEndpoint):
        def post(self, key_id: str, key_context: str = None):
            body = {
                'KeyId'     : key_id,
                'KeyContext': key_context
            }

            class Output(WebSdkOutputModel):
                fingerprint: str = ApiField(alias='Fingerprint')
                location: str = ApiField(alias='Location')
                public_key: str = ApiField(alias='PublicKey')
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetObjects(WebSdkEndpoint):
        def post(
            self, environment_filter: list[int] = None, include_chains: bool = None, include_archived: bool = None,
            key_id: str = None, key_context: str = None, object_type_filter: list[int] = None
        ):
            body = {
                'EnvironmentFilter': environment_filter,
                'IncludeChains'    : include_chains,
                'IncludeArchived'  : include_archived,
                'KeyId'            : key_id,
                'KeyContext'       : key_context,
                'ObjectTypeFilter' : object_type_filter,
            }

            class Output(WebSdkOutputModel):
                certificates: list[hsm_api.Certificate] = ApiField(alias='Certificates', default_factory=list)
                pending: bool = ApiField(alias='Pending')
                private_keys: list[hsm_api.PrivateKey] = ApiField(alias='PrivateKeys', default_factory=list)
                public_keys: list[hsm_api.PublicKey] = ApiField(alias='PublicKeys', default_factory=list)
                success: bool = ApiField(alias='Success')

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import system_status

class _SystemStatus(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/SystemStatus')
        self.Upgrade = self._Upgrade(api_obj=api_obj, url=f'{self._url}/Upgrade')
        self.Version = self._Version(api_obj=api_obj, url=f'{self._url}/Version')

    def get(self):
        class Output(WebSdkOutputModel):
            engines: list[system_status.SystemStatus] = ApiField(default_factory=list)

        return generate_output(output_cls=Output, response=self._get(), root_field='engines')

    class _Upgrade(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.Engine = self._Engine(api_obj=self._api_obj, url=f'{self._url}/Engine')
            self.Engines = self._Engines(api_obj=self._api_obj, url=f'{self._url}/Engines')
            self.History = self._History(api_obj=self._api_obj, url=f'{self._url}/History')
            self.Status = self._Status(api_obj=self._api_obj, url=f'{self._url}/Status')
            self.Summary = self._Summary(api_obj=self._api_obj, url=f'{self._url}/Summary')

        class _Engine(WebSdkEndpoint):
            def get(self, guid: str, engine_id: str, upgrade_id: str = None):
                params = {
                    'guid'     : guid,
                    'Id'       : engine_id,
                    'UpgradeId': upgrade_id
                }

                class Output(WebSdkOutputModel):
                    engine: system_status.Engine = ApiField(alias='Engine')
                    status: str = ApiField(alias='Status')
                    upgrade_start_time: datetime = ApiField(alias='UpgradeStartTime')
                    upgrade_stop_time: datetime = ApiField(alias='UpgradeStopTime')
                    tasks_completed: list[system_status.Task] = ApiField(default_factory=list, alias='TasksCompleted')
                    tasks_pending: list[system_status.Task] = ApiField(default_factory=list, alias='TasksPending')
                    tasks_running: list[system_status.Task] = ApiField(default_factory=list, alias='TasksRunning')

                return generate_output(output_cls=Output, response=self._get(params=params))

        class _Engines(WebSdkEndpoint):
            def get(self, upgrade_id: int = None):
                params = {
                    'UpgradeId': upgrade_id
                }

                class Output(WebSdkOutputModel):
                    engines: list[system_status.UpgradeStatus] = ApiField(default_factory=list, alias='Engines')

                return generate_output(output_cls=Output, response=self._get(params=params))

        class _History(WebSdkEndpoint):
            def get(self):
                class Output(WebSdkOutputModel):
                    upgrade_history: list[system_status.UpgradeInfo] = ApiField(
                        default_factory=list,
                        alias='UpgradeHistory'
                    )

                return generate_output(output_cls=Output, response=self._get())

        class _Status(WebSdkEndpoint):
            def get(self):
                class Output(WebSdkOutputModel):
                    upgrade_in_progress: bool = ApiField(alias='UpgradeInProgress')

                return generate_output(output_cls=Output, response=self._get())

        class _Summary(WebSdkEndpoint):
            def get(self):
                class Output(WebSdkOutputModel):
                    upgrade_summary: system_status.UpgradeSummary = ApiField(alias='UpgradeSummary')

                return generate_output(output_cls=Output, response=self._get())

    class _Version(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                version: str = ApiField(alias='Version')

            return generate_output(response=self._get(), output_cls=Output)



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class LogChannelContainerAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Log Channel Container"
    description = Attribute('Description')
    factory_item_vault_id = Attribute('Factory Item Vault Id')
    factory_vault_id = Attribute('Factory Vault Id')
    item_vault_id = Attribute('Item Vault Id')
    template = Attribute('Template')
    template_vault_id = Attribute('Template Vault Id')



from __future__ import annotations

from typing import (
    Any,
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import ssh

class _SSH(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/SSH')
        self.AddAuthorizedKey = self._AddAuthorizedKey(api_obj=self._api_obj, url=f'{self._url}/AddAuthorizedKey')
        self.AddHostPrivateKey = self._AddHostPrivateKey(api_obj=self._api_obj, url=f'{self._url}/AddHostPrivateKey')
        self.AddKnownHostKey = self._AddKnownHostKey(api_obj=self._api_obj, url=f'{self._url}/AddKnownHostKey')
        self.AddSelfServiceAuthorizedKey = self._AddSelfServiceAuthorizedKey(
            api_obj=self._api_obj,
            url=f'{self._url}/AddSelfServiceAuthorizedKey'
        )
        self.AddSelfServicePrivateKey = self._AddSelfServicePrivateKey(
            api_obj=self._api_obj,
            url=f'{self._url}/AddSelfServicePrivateKey'
        )
        self.AddUserPrivateKey = self._AddUserPrivateKey(api_obj=self._api_obj, url=f'{self._url}/AddUserPrivateKey')
        self.ApproveKeyOperation = self._ApproveKeyOperation(
            api_obj=self._api_obj,
            url=f'{self._url}/ApproveKeyOperation'
        )
        self.CancelKeyOperation = self._CancelKeyOperation(api_obj=self._api_obj, url=f'{self._url}/CancelKeyOperation')
        self.CancelRotation = self._CancelRotation(api_obj=self._api_obj, url=f'{self._url}/CancelRotation')
        self.ChangePrivateKeyPassphrase = self._ChangePrivateKeyPassphrase(
            api_obj=self._api_obj,
            url=f'{self._url}/ChangePrivateKeyPassphrase'
        )
        self.ConfirmSelfServiceKeyInstallation = self._ConfirmSelfServiceKeyInstallation(
            api_obj=self._api_obj,
            url=f'{self._url}/ConfirmSelfServiceKeyInstallation'
        )
        self.DeleteUnmatchedKeyset = self._DeleteUnmatchedKeyset(
            api_obj=self._api_obj,
            url=f'{self._url}/DeleteUnmatchedKeyset'
        )
        self.Devices = self._Devices(api_obj=self._api_obj, url=f'{self._url}/Devices')
        self.EditKeyOptions = self._EditKeyOptions(api_obj=self._api_obj, url=f'{self._url}/EditKeyOptions')
        self.EditSelfServiceAuthorizedKey = self._EditSelfServiceAuthorizedKey(
            api_obj=self._api_obj,
            url=f'{self._url}/EditSelfServiceAuthorizedKey'
        )
        self.ExportSelfServiceAuthorizedKey = self._ExportSelfServiceAuthorizedKey(
            api_obj=self._api_obj,
            url=f'{self._url}/ExportSelfServiceAuthorizedKey'
        )
        self.ExportSelfServicePrivateKey = self._ExportSelfServicePrivateKey(
            api_obj=self._api_obj,
            url=f'{self._url}/ExportSelfServicePrivateKey'
        )
        self.ImportAuthorizedKey = self._ImportAuthorizedKey(
            api_obj=self._api_obj,
            url=f'{self._url}/ImportAuthorizedKey'
        )
        self.ImportKeyUsageData = self._ImportKeyUsageData(api_obj=self._api_obj, url=f'{self._url}/ImportKeyUsageData')
        self.ImportPrivateKey = self._ImportPrivateKey(api_obj=self._api_obj, url=f'{self._url}/ImportPrivateKey')
        self.KeyDetails = self._KeyDetails(api_obj=self._api_obj, url=f'{self._url}/KeyDetails')
        self.KeysetDetails = self._KeysetDetails(api_obj=self._api_obj, url=f'{self._url}/KeysetDetails')
        self.KeyUsage = self._KeyUsage(api_obj=self._api_obj, url=f'{self._url}/KeyUsage')
        self.MoveKeysetsToPolicy = self._MoveKeysetsToPolicy(
            api_obj=self._api_obj,
            url=f'{self._url}/MoveKeysetsToPolicy'
        )
        self.RejectKeyOperation = self._RejectKeyOperation(api_obj=self._api_obj, url=f'{self._url}/RejectKeyOperation')
        self.RemoveKey = self._RemoveKey(api_obj=self._api_obj, url=f'{self._url}/RemoveKey')
        self.RetryKeyOperation = self._RetryKeyOperation(api_obj=self._api_obj, url=f'{self._url}/RetryKeyOperation')
        self.RetryRotation = self._RetryRotation(api_obj=self._api_obj, url=f'{self._url}/RetryRotation')
        self.RollbackNonBlocking = self._RollbackNonBlocking(
            api_obj=self._api_obj,
            url=f'{self._url}/RollbackNonBlocking'
        )
        self.RotateNonBlocking = self._RotateNonBlocking(api_obj=self._api_obj, url=f'{self._url}/RotateNonBlocking')
        self.Rotate = self._Rotate(api_obj=self._api_obj, url=f'{self._url}/Rotate')
        self.SetUnmatchedKeysetPassPhrase = self._SetUnmatchedKeysetPassPhrase(
            api_obj=self._api_obj,
            url=f'{self._url}/SetUnmatchedKeysetPassPhrase'
        )
        self.SkipKeyRotation = self._SkipKeyRotation(api_obj=self._api_obj, url=f'{self._url}/SkipKeyRotation')
        self.TestDeviceConnection = self._TestDeviceConnection(
            api_obj=self._api_obj,
            url=f'{self._url}/TestDeviceConnection'
        )
        self.Widget = self._Widget(api_obj=self._api_obj, url=f'{self._url}/Widget')

    class _AddAuthorizedKey(WebSdkEndpoint):
        # noinspection ALL
        def post(
            self,
            device_guid: str,
            filepath: str,
            keyset_id: str,
            username: str,
            allowed_source_restriction: list[str] = None,
            denied_source_restriction: list[str] = None,
            forced_command: str = None,
            format: str = None,
            options: list[str] = None
        ):
            body = {
                'AllowedSourceRestriction': allowed_source_restriction,
                'DeniedSourceRestriction' : denied_source_restriction,
                'DeviceGuid'              : device_guid,
                'Filepath'                : filepath,
                'ForcedCommand'           : forced_command,
                'Format'                  : format,
                'KeysetId'                : keyset_id,
                'Options'                 : options,
                'Username'                : username
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddHostPrivateKey(WebSdkEndpoint):
        # noinspection ALL
        def post(self, device_guid: str, filepath: str, username: str, format: str = None, policy_dn: str = None):
            body = {
                'DeviceGuid': device_guid,
                'Filepath'  : filepath,
                'Format'    : format,
                'Username'  : username,
                'PolicyDN'  : policy_dn
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                keyset_id: str = ApiField(alias='KeysetId')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddKnownHostKey(WebSdkEndpoint):
        # noinspection ALL
        def post(self, device_guid: str, filepath: str, keyset_id: str, username: str, format: str = None):
            body = {
                'DeviceGuid': device_guid,
                'Filepath'  : filepath,
                'Format'    : format,
                'KeysetId'  : keyset_id,
                'Username'  : username
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddSelfServiceAuthorizedKey(WebSdkEndpoint):
        def post(
            self,
            allowed_source_restriction: list[str],
            denied_source_restriction: list[str],
            folder_id: str,
            location: str,
            notes: str,
            options: list[str],
            owner: str,
            contact_email: str = None,
            forced_command: str = None,
            keyset_id: str = None
        ):
            body = {
                'AllowedSourceRestriction': allowed_source_restriction,
                'ContactEmail'            : contact_email,
                'DeniedSourceRestriction' : denied_source_restriction,
                'FolderId'                : folder_id,
                'ForcedCommand'           : forced_command,
                'KeysetId'                : keyset_id,
                'Location'                : location,
                'Notes'                   : notes,
                'Options'                 : options,
                'Owner'                   : owner
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                keyset_id: str = ApiField(alias='KeysetId')
                notes: str = ApiField(alias='Notes')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddSelfServicePrivateKey(WebSdkEndpoint):
        def post(
            self,
            folder_id: str,
            location: str,
            notes: str,
            owner: str,
            contact_email: str = None,
            keyset_id: str = None
        ):
            body = {
                'ContactEmail': contact_email,
                'FolderId'    : folder_id,
                'KeysetId'    : keyset_id,
                'Location'    : location,
                'Notes'       : notes,
                'Owner'       : owner
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                keyset_id: str = ApiField(alias='KeysetId')
                notes: str = ApiField(alias='Notes')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddUserPrivateKey(WebSdkEndpoint):
        # noinspection ALL
        def post(
            self, device_guid: str, filepath: str, username: str, format: str = None, keyset_id: str = None,
            passphrase: str = None, policy_dn: str = None
        ):
            body = {
                'DeviceGuid': device_guid,
                'Filepath'  : filepath,
                'Format'    : format,
                'KeysetId'  : keyset_id,
                'Passphrase': passphrase,
                'Username'  : username,
                'PolicyDN'  : policy_dn
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                keyset_id: str = ApiField(alias='KeysetId')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ApproveKeyOperation(WebSdkEndpoint):
        def post(self, key_id: int, comment: str):
            body = {
                'KeyId'  : key_id,
                'Comment': comment
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CancelKeyOperation(WebSdkEndpoint):
        def post(self, key_id: int):
            body = {
                'KeyId': key_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CancelRotation(WebSdkEndpoint):
        def post(self, keyset_id: str):
            body = {
                'KeysetId': keyset_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ChangePrivateKeyPassphrase(WebSdkEndpoint):
        def post(self, key_id: int, passphrase: str):
            body = {
                'KeysetId'  : key_id,
                'Passphrase': passphrase
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ConfirmSelfServiceKeyInstallation(WebSdkEndpoint):
        def post(self, keyset_id: str):
            body = {
                'KeysetId': keyset_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _DeleteUnmatchedKeyset(WebSdkEndpoint):
        def post(self, unmatched_trust_id: str):
            body = {
                'UnmatchedTrustId': unmatched_trust_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Devices(WebSdkEndpoint):
        def post(self, page_size: int, offset: int = None, ssh_device_filter: Union[dict, ssh.SshDeviceFilter] = None):
            body = {
                'PageSize'       : page_size,
                'Offset'         : offset,
                'SshDeviceFilter': ssh_device_filter
            }

            class Output(WebSdkOutputModel):
                data: list[ssh.DeviceData] = ApiField(default_factory=list, alias='Data')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _EditKeyOptions(WebSdkEndpoint):
        def post(
            self,
            key_id: int,
            allowed_source_restriction: list[str] = None,
            denied_source_restriction: list[str] = None,
            forced_command: str = None,
            options: list[str] = None
        ):
            body = {
                'AllowedSourceRestriction': allowed_source_restriction,
                'DeniedSourceRestriction' : denied_source_restriction,
                'ForcedCommand'           : forced_command,
                'KeyId'                   : key_id,
                'Options'                 : options
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _EditSelfServiceAuthorizedKey(WebSdkEndpoint):
        def post(
            self, key_id: int, allowed_source_restriction: list[str] = None,
            denied_source_restriction: list[str] = None, forced_command: str = None,
            location: str = None, notes: str = None, options: list[str] = None
        ):
            body = {
                'AllowedSourceRestriction': allowed_source_restriction,
                'DeniedSourceRestriction' : denied_source_restriction,
                'ForcedCommand'           : forced_command,
                'KeyId'                   : key_id,
                'Location'                : location,
                'Notes'                   : notes,
                'Options'                 : options
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ExportSelfServiceAuthorizedKey(WebSdkEndpoint):
        # noinspection ALL
        def post(self, key_id: int, format: str = None):
            body = {
                'KeyId' : key_id,
                'Format': format
            }

            class Output(WebSdkOutputModel):
                key_material: str = ApiField(alias='KeyMaterial')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ExportSelfServicePrivateKey(WebSdkEndpoint):
        # noinspection ALL
        def post(self, key_id: int, format: str = None, passphrase: str = None):
            body = {
                'KeyId'     : key_id,
                'Format'    : format,
                'Passphrase': passphrase
            }

            class Output(WebSdkOutputModel):
                key_material: str = ApiField(alias='KeyMaterial')
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ImportAuthorizedKey(WebSdkEndpoint):
        # noinspection ALL
        def post(self, device_guid: str, filepath: str, format: str, key_content_base_64: str, username: str):
            body = {
                'DeviceGuid'      : device_guid,
                'Filepath'        : filepath,
                'Format'          : format,
                'KeyContentBase64': key_content_base_64,
                'Username'        : username
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                response: ssh.SshWebResponse = ApiField(alias='SshWebResponse')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ImportKeyUsageData(WebSdkEndpoint):
        def post(self, log_data: list[ssh.LogData]):
            body = {
                'LogData': log_data
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='SshWebResponse')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ImportPrivateKey(WebSdkEndpoint):
        # noinspection ALL
        def post(
            self, device_guid: str, filepath: str, format: str, key_content_base_64: str, username: str,
            passphrase: str = None
        ):
            body = {
                'DeviceGuid'      : device_guid,
                'Filepath'        : filepath,
                'Format'          : format,
                'KeyContentBase64': key_content_base_64,
                'Passphrase'      : passphrase,
                'Username'        : username
            }

            class Output(WebSdkOutputModel):
                key_id: int = ApiField(alias='KeyId')
                response: ssh.SshWebResponse = ApiField(alias='SshWebResponse')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _KeyDetails(WebSdkEndpoint):
        def post(self, key_id: int):
            body = {
                'KeyId': key_id
            }

            class Output(WebSdkOutputModel):
                key_data: list[ssh.KeyData] = ApiField(default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body), root_field='key_data')

    class _KeysetDetails(WebSdkEndpoint):
        def get(self, keyset_id: str, load_key_data: bool = None):
            params = {
                'KeysetId'   : keyset_id,
                'LoadKeyData': load_key_data
            }

            class Output(WebSdkOutputModel, ssh.KeySetData):
                pass

            return generate_output(output_cls=Output, response=self._get(params=params))

        def post(
            self, page_size: int, keyset_filter: list[ssh.KeySetFilter] = None, load_key_data: bool = None,
            offset: int = None
        ):
            body = {
                'KeysetFilter': keyset_filter,
                'LoadKeyData' : load_key_data,
                'Offset'      : offset,
                'PageSize'    : page_size
            }

            class Output(WebSdkOutputModel):
                data: list[ssh.KeySetData] = ApiField(default_factory=list, alias='Data')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _KeyUsage(WebSdkEndpoint):
        def post(self, ssh_key_usage_filter: list[ssh.KeyUsageFilter], page_size: int = None, offset: int = None):
            body = {
                'SshKeyUsageFilter': ssh_key_usage_filter,
                'PageSize'         : page_size,
                'Offset'           : offset
            }

            class Output(WebSdkOutputModel):
                data: list[ssh.KeyUsageData] = ApiField(default_factory=list, alias='Data')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _MoveKeysetsToPolicy(WebSdkEndpoint):
        def post(self, keyset_ids: list[str], policy_dn: str = None, policy_path: str = None):
            body = {
                'KeysetIds' : keyset_ids,
                'PolicyDN'  : policy_dn,
                'PolicyPath': policy_path
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RejectKeyOperation(WebSdkEndpoint):
        def post(self, key_id: int, comment: str):
            body = {
                'KeyId'  : key_id,
                'Comment': comment
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RemoveKey(WebSdkEndpoint):
        def post(self, key_id: int):
            body = {
                'KeyId': key_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RetryKeyOperation(WebSdkEndpoint):
        def post(self, key_id: int):
            body = {
                'KeyId': key_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RetryRotation(WebSdkEndpoint):
        def post(self, keyset_id: str):
            body = {
                'KeysetId': keyset_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RollbackNonBlocking(WebSdkEndpoint):
        def post(self, keyset_id: str):
            body = {
                'KeysetId': keyset_id
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                error_code: int = ApiField(alias='ErrorCode')
                error_message: str = ApiField(alias='ErrorMessage')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Rotate(WebSdkEndpoint):
        def post(self, keyset_id: str, options: int = None, allow_skip_on_rotation: bool = None):
            body = {
                'KeysetId'           : keyset_id,
                'AllowSkipOnRotation': allow_skip_on_rotation,
                'Options'            : options
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RotateNonBlocking(WebSdkEndpoint):
        def post(self, keyset_id: str, options: int = None, allow_skip_on_rotation: bool = None):
            body = {
                'KeysetId'           : keyset_id,
                'AllowSkipOnRotation': allow_skip_on_rotation,
                'Options'            : options
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                error_code: int = ApiField(alias='ErrorCode')
                error_message: str = ApiField(alias='ErrorMessage')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _SetUnmatchedKeysetPassPhrase(WebSdkEndpoint):
        def post(self, passphrase: str, unmatched_trust_id: str):
            body = {
                'Passphrase'      : passphrase,
                'UnmatchedTrustId': unmatched_trust_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _SkipKeyRotation(WebSdkEndpoint):
        def post(self, key_id: int):
            body = {
                'KeyId': key_id
            }

            class Output(WebSdkOutputModel):
                response: ssh.SshWebResponse = ApiField(alias='Response')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _TestDeviceConnection(WebSdkEndpoint):
        def post(self, device_guids: list[str]):
            body = {
                'deviceGuids': device_guids
            }

            class Output(WebSdkOutputModel):
                connection_results: list[ssh.ConnectionResult] = ApiField(default_factory=list)

            return generate_output(output_cls=Output, response=self._post(data=body), root_field='connection_results')

    class _Widget(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.CriticalAlerts = self._CriticalAlerts(api_obj=self._api_obj, url=f'{self._url}/CriticalAlerts')
            self.PolicyViolations = self._PolicyViolations(api_obj=self._api_obj, url=f'{self._url}/PolicyViolations')
            self.Stats = self._Stats(api_obj=self._api_obj, url=f'{self._url}/Stats')

        class _CriticalAlerts(WebSdkEndpoint):
            def get(self, min_allowed_key_length: int):
                params = {
                    'minAllowedKeyLength': min_allowed_key_length
                }

                class Output(WebSdkOutputModel):
                    accessible_root_accounts: int = ApiField(alias='AccessibleRootAccounts')
                    non_compliant_duplicate_private_keys: int = ApiField(alias='NonCompliantDuplicatePrivateKeys')
                    non_root_orphans: int = ApiField(alias='NonRootOrphans')
                    root_orphans: int = ApiField(alias='RootOrphans')
                    shared_private_keys: int = ApiField(alias='SharedPrivateKeys')
                    very_small_key: int = ApiField(alias='VerySmallKey')

                return generate_output(output_cls=Output, response=self._get(params=params))

        class _PolicyViolations(WebSdkEndpoint):
            def get(self):
                class Output(WebSdkOutputModel):
                    duplicate_private_keys: list[str] = ApiField(alias='DuplicatePrivateKeys', default_factory=list)
                    key_older_than_policy: str = ApiField(alias='KeyOlderThanPolicy')
                    key_smaller_than_policy: str = ApiField(alias='KeySmallerThanPolicy')
                    missing_options: list[str] = ApiField(alias='MissingOptions', default_factory=list)
                    non_compliant_algorithm: str = ApiField(alias='NonCompliantAlgorithm')
                    non_compliant_force_command: str = ApiField(alias='NonCompliantForceCommand')
                    non_compliant_source_restriction: str = ApiField(alias='NonCompliantSourceRestriction')
                    non_compliant_vendor_format: str = ApiField(alias='NonCompliantVendorFormat')
                    root_access: str = ApiField(alias='RootAccess')
                    shared_server_account: str = ApiField(alias='SharedServerAccount')
                    vulnerable_protocol: str = ApiField(alias='VulnerableProtocol')

                return generate_output(output_cls=Output, response=self._get())

        class _Stats(WebSdkEndpoint):
            def get(self, group_by: str):
                params = {
                    'GroupBy': group_by
                }

                class Output(WebSdkOutputModel):
                    stats: dict[str, Any] = ApiField()

                return generate_output(
                    output_cls=Output, response=self._get(params=params),
                    root_field='stats'
                )



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class StatisticsCounterAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Statistics Counter"
    sensitive = Attribute('Sensitive')
    statistic_id = Attribute('Statistic Id')
    tag_a_name = Attribute('Tag A Name')
    tag_b_name = Attribute('Tag B Name')
    tag_c_name = Attribute('Tag C Name')
    value_description = Attribute('Value Description')



from __future__ import annotations

import base64
import hashlib
from typing import (
    TYPE_CHECKING,
    Union,
)

from pyvenafi.tpp.api.websdk.enums.secret_store import (
    KeyNames,
    Namespaces,
    VaultTypes,
)
from pyvenafi.tpp.attributes.adaptable_ca import AdaptableCAAttributes
from pyvenafi.tpp.attributes.microsoft_ca import MicrosoftCAAttributes
from pyvenafi.tpp.attributes.self_signed_ca import SelfSignedCAAttributes
from pyvenafi.tpp.features.bases.feature_base import (
    feature,
    FeatureBase,
)

if TYPE_CHECKING:
    from pyvenafi.tpp.api.websdk.models import (
        config,
        identity as ident,
    )

class _CertificateAuthorityBase(FeatureBase):
    def __init__(self, api):
        super().__init__(api=api)

    def delete(self, certificate_authority: 'Union[config.Object, str]'):
        """
        Deletes the certificate authority object from TPP, including all of the secrets associated to it.

        Args:
            certificate_authority: :ref:`config_object` or :ref:`dn` for the certificate authority object.
        """
        certificate_authority_dn = self._get_dn(certificate_authority)
        self._secret_store_delete(object_dn=certificate_authority_dn)
        self._config_delete(object_dn=certificate_authority_dn)

    def get(self, certificate_authority_dn: str, raise_error_if_not_exists: bool = True):
        """
        Get the certificate authority object in TPP.

        Args:
            certificate_authority_dn: :ref:`dn` of the certificate authority.
            raise_error_if_not_exists: Raise an exception if the object :ref:`dn` does not exist.

        Returns:
            :ref:`config_object` of the certificate authority.
        """
        return self._get_config_object(
            object_dn=certificate_authority_dn,
            raise_error_if_not_exists=raise_error_if_not_exists
        )

@feature('Microsoft CA')
class MSCA(_CertificateAuthorityBase):
    def __init__(self, api):
        super().__init__(api=api)

    def create(
        self, name: str, parent_folder: 'Union[config.Object, str]', hostname: str, service_name: str,
        credential: 'Union[config.Object, str]', template: str, description: 'str' = None,
        contacts: 'list[ident.Identity, str]' = None, manual_approvals: 'bool' = None,
        subject_alt_name_enabled: 'bool' = None, automatically_include_cn_as_dns_san: 'bool' = None,
        allow_users_to_specify_end_date: 'bool' = None, enrollment_agent: 'Union[config.Object, str]' = None,
        attributes: dict = None, get_if_already_exists: bool = True
    ):
        """
        Args:
            name: Name of the CA object.
            parent_folder: `:ref:`config_object` or :ref:`dn` of the parent folder of this certificate authority object.
            hostname: Hostname or IP Address of the CA.
            service_name: Service, or Given, Name of the certificate authority.
            credential: `:ref:`config_object` or :ref:`dn` of the CA credential.
            template: Name of the CA template.
            description: Description of the CA object.
            contacts: List of :ref:`identity_object` or :ref:`prefixed_name` of the contacts.
            manual_approvals: Require manual approvals.
            subject_alt_name_enabled: Enable Subject Alternative Names.
            automatically_include_cn_as_dns_san: Automatically include the common name (CN) as a DNS SAN.
            allow_users_to_specify_end_date: Allow users to specify the end date.
            enrollment_agent: `:ref:`config_object` or :ref:`dn` of the certificate credential, or enrollment agent.
            attributes: Additional attributes associated to the CA object.
            get_if_already_exists: If the objects already exists, just return it as is.

        Returns:
            :ref:`config_object` of the certificate authority.
        """
        bool_to_string = lambda x: {
            True : "1",
            False: "0"
        }.get(x)
        ca_attrs = {
            MicrosoftCAAttributes.driver_name                 : 'camicrosoft',
            MicrosoftCAAttributes.host                        : hostname,
            MicrosoftCAAttributes.given_name                  : service_name,
            MicrosoftCAAttributes.credential                  : self._get_dn(credential),
            MicrosoftCAAttributes.template                    : template,
            MicrosoftCAAttributes.description                 : description,
            MicrosoftCAAttributes.contact                     : [self._get_prefixed_universal(c) for c in
                                                                 contacts] if contacts else None,
            MicrosoftCAAttributes.manual_approval             : bool_to_string(manual_approvals),
            MicrosoftCAAttributes.san_enabled                 : bool_to_string(subject_alt_name_enabled),
            MicrosoftCAAttributes.include_cn_as_san           : bool_to_string(automatically_include_cn_as_dns_san),
            MicrosoftCAAttributes.specific_end_date_enabled   : bool_to_string(allow_users_to_specify_end_date),
            MicrosoftCAAttributes.enrollment_agent_certificate: self._get_dn(
                enrollment_agent
            ) if enrollment_agent else None
        }
        if attributes:
            ca_attrs.update(attributes)

        return self._config_create(
            name=name,
            parent_folder_dn=self._get_dn(parent_folder),
            config_class=MicrosoftCAAttributes.__config_class__,
            attributes=ca_attrs,
            get_if_already_exists=get_if_already_exists
        )

@feature('Self-Signed CA')
class SelfSignedCA(_CertificateAuthorityBase):
    def __init__(self, api):
        super().__init__(api=api)

    def create(
        self, name: str, parent_folder: 'Union[config.Object, str]', description: 'str' = None,
        contacts: 'list[ident.Identity, str]' = None, key_usage: 'list[str]' = None,
        server_authentication: 'bool' = None, client_authentication: 'bool' = None, code_signing: 'bool' = None,
        time_stamping: 'bool' = None, signature_algorithm: 'str' = None, valid_years: 'int' = None,
        valid_days: 'int' = None, attributes: dict = None, get_if_already_exists: bool = True
    ):
        """
        Args:
            name: Name of the CA object.
            parent_folder: `:ref:`config_object` or :ref:`dn` of the parent folder of this certificate authority object.
            description: Description of the CA object.
            contacts: List of :ref:`identity_object` or :ref:`prefixed_name` of the contacts.
            key_usage: List of key usages.
            server_authentication: Allow server authentication.
            client_authentication: Allow client authentication.
            code_signing: Allow code signing.
            time_stamping: Allow time stamping.
            signature_algorithm: Signing algorithm.
            valid_years: Validity period in years.
            valid_days: Validity period in days. Added to years.
            attributes: Additional attributes associated to the CA object.
            get_if_already_exists: If the objects already exists, just return it as is.

        Returns:
            :ref:`config_object` of the certificate authority.
        """
        ca_attrs = {
            SelfSignedCAAttributes.driver_name: 'caselfsigned',
            SelfSignedCAAttributes.description: description,
            SelfSignedCAAttributes.contact    : [self._get_prefixed_universal(c) for c in
                                                 contacts] if contacts else None,
            SelfSignedCAAttributes.key_usage  : ','.join(key_usage),
            SelfSignedCAAttributes.algorithm  : signature_algorithm,
        }
        if server_authentication or client_authentication or code_signing or time_stamping:
            enhanced_key_usage = {
                '1.3.6.1.5.5.7.3.1': server_authentication,
                '1.3.6.1.5.5.7.3.2': client_authentication,
                '1.3.6.1.5.5.7.3.3': code_signing,
                '1.3.6.1.5.5.7.3.8': time_stamping
            }
            ca_attrs.update(
                {
                    SelfSignedCAAttributes.enhanced_key_usage: [
                        eku for eku, enabled in enhanced_key_usage.items() if enabled is True
                    ],
                }
            )
        if valid_years or valid_days:
            validity_period = (365 * (valid_years or 0)) + (valid_days or 0)
            ca_attrs.update(
                {
                    SelfSignedCAAttributes.validity_period: validity_period
                }
            )
        if attributes:
            ca_attrs.update(attributes)
        return self._config_create(
            name=name,
            parent_folder_dn=self._get_dn(parent_folder),
            config_class=SelfSignedCAAttributes.__config_class__,
            attributes=ca_attrs,
            get_if_already_exists=get_if_already_exists
        )

@feature('Self-Signed CA')
class AdaptableCA(_CertificateAuthorityBase):
    def __init__(self, api):
        super().__init__(api=api)

    def create(
        self, name: str, parent_folder: 'Union[config.Object, str]', powershell_script_name: str,
        powershell_script_content: bytes, oauth_application_id: str,
        oauth_token_credential: 'Union[config.Object, str]', description: 'str' = None,
        contacts: 'list[ident.Identity, str]' = None, username_credential: 'Union[config.Object, str]' = None,
        certificate_credential: 'Union[config.Object, str]' = None,
        secondary_credential: 'Union[config.Object, str]' = None, service_address: 'str' = None,
        profile_string: 'str' = None, renewal_window: int = None,
        allow_reissuance: bool = False, subject_alt_name_enabled: bool = False,
        fix_certificate_errors: bool = False, enabled_debug_logging: bool = False,
        oauth_scope: str = None, attributes: dict = None, get_if_already_exists: bool = True
    ):
        """
        Args:
            name: Name of the CA object.
            parent_folder: `:ref:`config_object` or :ref:`dn` of the parent folder of this certificate authority object.
            powershell_script_name: Powershell script name on the TPP server.
            powershell_script_content: Powershell script content on the TPP server.
            oauth_application_id: Application ID of the OAuth application.
            oauth_token_credential: `:ref:`config_object` or :ref:`dn` of the parent folder of this certificate authority object.
            description: Description of the CA object.
            contacts: List of :ref:`identity_object` or :ref:`prefixed_name` of the contacts.
            username_credential: `:ref:`config_object` or :ref:`dn` of a username credential.
            certificate_credential: `:ref:`config_object` or :ref:`dn` of a certificate credential.
            secondary_credential: `:ref:`config_object` or :ref:`dn` of a secondary credential.
            service_address: Any service address.
            profile_string: Profile string.
            renewal_window: Renewal window of the CA object.
            allow_reissuance: Allow certificate reissuance.
            subject_alt_name_enabled: Enable subject alternate names.
            fix_certificate_errors: Fix certificate errors when the script is updated.
            enabled_debug_logging: Enable debug logging.
            oauth_scope: Scope of the OAuth application.
            attributes: Additional attributes associated to the CA object.
            get_if_already_exists: If the objects already exists, just return it as is.

        Returns:
            :ref:`config_object` of the certificate authority.
        """
        bool_to_str = lambda x: "1" if x else None
        ca_attrs = {
            AdaptableCAAttributes.driver_name                     : 'caadaptable',
            AdaptableCAAttributes.description                     : description,
            AdaptableCAAttributes.contact                         : [
                self._get_prefixed_universal(c) for c in
                contacts
            ] if contacts else None,
            AdaptableCAAttributes.credential                      : self._get_dn(username_credential),
            AdaptableCAAttributes.certificate_credential          : self._get_dn(certificate_credential),
            AdaptableCAAttributes.secondary_credential            : self._get_dn(secondary_credential),
            AdaptableCAAttributes.service_address                 : service_address,
            AdaptableCAAttributes.profile_string                  : profile_string,
            AdaptableCAAttributes.renewal_window                  : renewal_window,
            AdaptableCAAttributes.allow_reissue                   : bool_to_str(allow_reissuance),
            AdaptableCAAttributes.san_enabled                     : bool_to_str(subject_alt_name_enabled),
            AdaptableCAAttributes.retry_after_script_hash_mismatch: bool_to_str(fix_certificate_errors),
            AdaptableCAAttributes.log_debug                       : bool_to_str(enabled_debug_logging),
            AdaptableCAAttributes.oauth_token_application_id      : oauth_application_id,
            AdaptableCAAttributes.oauth_token_credential          : self._get_dn(oauth_token_credential),
            AdaptableCAAttributes.oauth_token_scope               : oauth_scope,
        }
        if attributes:
            ca_attrs.update(attributes)
        ca = self._config_create(
            name=name,
            parent_folder_dn=self._get_dn(parent_folder),
            config_class=AdaptableCAAttributes.__config_class__,
            attributes=ca_attrs,
            get_if_already_exists=get_if_already_exists
        )

        self._api.websdk.Config.AddValue.post(
            object_dn=ca.dn,
            attribute_name=AdaptableCAAttributes.interoperability_script,
            value=powershell_script_name
        )

        script_hash = base64.b64encode(
            hashlib.sha256(
                powershell_script_content.decode().encode('utf-32-le')
            ).hexdigest().encode()
        ).decode()

        vault_id = self._api.websdk.SecretStore.Add.post(
            base_64_data=script_hash,
            keyname=KeyNames.software_default,
            vault_type=VaultTypes.blob,
            namespace=Namespaces.config,
            owner=ca.dn,
        ).vault_id

        self._api.websdk.Config.WriteDn.post(
            object_dn=ca.dn,
            attribute_name=AdaptableCAAttributes.powershell_script_hash_vault_id,
            values=[vault_id]
        )

        return ca



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class TrustAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Trust"
    allow_from = Attribute('Allow From', min_version='21.4')
    allowed_algorithm = Attribute('Allowed Algorithm', min_version='21.4')
    allowed_command = Attribute('Allowed Command', min_version='21.4')
    allowed_vendor_types = Attribute('Allowed Vendor Types', min_version='21.4')
    automatic_rotation_enabled = Attribute('Automatic Rotation Enabled', min_version='23.3')
    automatic_rotation_interval = Attribute('Automatic Rotation Interval', min_version='23.3')
    automatic_rotation_lead_time = Attribute('Automatic Rotation Lead Time', min_version='23.3')
    contact = Attribute('Contact', min_version='21.4')
    deny_from = Attribute('Deny From', min_version='21.4')
    key_bit_strength = Attribute('Key Bit Strength', min_version='21.4')
    management_type = Attribute('Management Type', min_version='23.3')
    maximum_authorizations_per_keyset = Attribute('Maximum Authorizations Per Keyset', min_version='21.4')
    minimum_key_bit_strength = Attribute('Minimum Key Bit Strength', min_version='21.4')
    pbes2_algorithm = Attribute('PBES2 Algorithm', min_version='24.1')
    required_options = Attribute('Required Options', min_version='21.4')
    restricted_pbes2_algorithms = Attribute('Restricted PBES2 Algorithms', min_version='24.1')
    trust_id = Attribute('Trust Id', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes

class MicrosoftCAPoolAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Microsoft CA Pool"
    certificate_authority = Attribute('Certificate Authority', min_version='21.4')
    second_certificate_authority = Attribute('Second Certificate Authority', min_version='21.4')
    third_certificate_authority = Attribute('Third Certificate Authority', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes

class RSAKeonCAAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "RSA Keon CA"
    ca_md5 = Attribute('CA MD5', min_version='21.4')
    ca_name = Attribute('CA Name', min_version='21.4')
    certificate_block = Attribute('Certificate Block', min_version='21.4')
    jurisdiction_id = Attribute('Jurisdiction ID', min_version='21.4')
    jurisdiction_name = Attribute('Jurisdiction Name', min_version='21.4')
    supported_validity_periods = Attribute('Supported Validity Periods', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class VAMnShieldAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "VAM nShield"
    file_validation_disabled = Attribute('File Validation Disabled', min_version='15.3')
    install_path = Attribute('Install Path')
    km_local_path = Attribute('KM Local Path')
    key_store_validation_disabled = Attribute('Key Store Validation Disabled')
    module_id = Attribute('Module Id')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.code_signing_environment_base import CodeSigningEnvironmentBaseAttributes

class CodeSigningGPGEnvironmentAttributes(CodeSigningEnvironmentBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing GPG Environment"
    authentication_key_algorithm = Attribute('Authentication Key Algorithm', min_version='21.4')
    authentication_key_dn = Attribute('Authentication Key DN', min_version='21.4')
    encryption_key_algorithm = Attribute('Encryption Key Algorithm', min_version='21.4')
    encryption_key_dn = Attribute('Encryption Key DN', min_version='21.4')
    internet_email_address = Attribute('Internet EMail Address', min_version='21.4')
    is_issuer = Attribute('Is Issuer', min_version='22.3')
    issuer_environment_dn = Attribute('Issuer Environment DN', min_version='22.3')
    issuer_key_dn = Attribute('Issuer Key DN', min_version='21.4')
    key_storage_location = Attribute('Key Storage Location', min_version='21.4')
    max_uses = Attribute('Max Uses', min_version='21.4')
    real_name = Attribute('Real Name', min_version='21.4')
    signing_key_algorithm = Attribute('Signing Key Algorithm', min_version='21.4')
    signing_key_dn = Attribute('Signing Key DN', min_version='21.4')
    validity_period = Attribute('Validity Period', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.flow_action_base import FlowActionBaseAttributes

class FlowActionAddTeamMemberAttributes(FlowActionBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Flow Action Add Team Member"



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from typing import (
    Any,
    List,
    Literal,
)
from uuid import UUID

AnyValue = Any

class ConnectorProperties(ObjectModel):
    connectorKind: str = ApiField(alias='connectorKind')
    connectorType: Literal['WEBHOOK'] = ApiField(alias='connectorType')

class ConnectorsCreationRequest(ObjectModel):
    name: str = ApiField(alias='name')
    properties: ConnectorProperties = ApiField(alias='properties')

class ConnectorsInformation(ObjectModel):
    id: UUID = ApiField(alias='id')
    name: str = ApiField(alias='name')
    properties: ConnectorProperties = ApiField(alias='properties')

class ConnectorsResponse(ObjectModel):
    connectors: List[ConnectorsInformation] = ApiField(alias='connectors', default_factory=list)

class ConnectorsUpdateRequest(ObjectModel):
    name: str = ApiField(alias='name')
    properties: ConnectorProperties = ApiField(alias='properties')

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class Filter(ObjectModel):
    filterType: str = ApiField(alias='filterType')

class JsonNode(ObjectModel):
    ...

class Target(ObjectModel):
    connection: AnyValue = ApiField(alias='connection')
    type: str = ApiField(alias='type')

class WebhookProperties(ConnectorProperties):
    filter: Filter = ApiField(alias='filter')
    target: Target = ApiField(alias='target')

class ActivityFilter(Filter):
    activities: List[str] = ApiField(alias='activities', default_factory=list)
    activityTypes: List[str] = ApiField(alias='activityTypes', default_factory=list)
    criticality: int = ApiField(alias='criticality')

class ExpirationFilter(Filter):
    applicationIds: List[UUID] = ApiField(alias='applicationIds', default_factory=list)

ActivityFilter.update_forward_refs()
ConnectorProperties.update_forward_refs()
ConnectorsCreationRequest.update_forward_refs()
ConnectorsInformation.update_forward_refs()
ConnectorsResponse.update_forward_refs()
ConnectorsUpdateRequest.update_forward_refs()
ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
ExpirationFilter.update_forward_refs()
Filter.update_forward_refs()
JsonNode.update_forward_refs()
Target.update_forward_refs()
WebhookProperties.update_forward_refs()



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class DriverBaseAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Driver Base"
    driver_arguments = Attribute('Driver Arguments', min_version='21.4')
    driver_name = Attribute('Driver Name', min_version='21.4')
    rank = Attribute('Rank', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.service_module import ServiceModuleAttributes

class BulkProvisioningManagerAttributes(ServiceModuleAttributes, metaclass=IterableMeta):
    __config_class__ = "Bulk Provisioning Manager"
    retry_interval = Attribute('Retry Interval', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes
from pyvenafi.tpp.attributes.top import TopAttributes

class PlacementJobAttributes(ScheduleBaseAttributes, TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Placement Job"
    archive_old_certificates = Attribute('Archive Old Certificates', min_version='21.4')
    default_container = Attribute('Default Container', min_version='21.4')
    include_subfolders = Attribute('Include Subfolders', min_version='21.4')
    last_run = Attribute('Last Run', min_version='21.4')
    layout_rules = Attribute('Layout Rules', min_version='21.4')
    objects_combined = Attribute('Objects Combined', min_version='21.4')
    objects_evaluated = Attribute('Objects Evaluated', min_version='21.4')
    objects_moved = Attribute('Objects Moved', min_version='21.4')
    progress = Attribute('Progress', min_version='21.4')
    rules_order = Attribute('Rules Order', min_version='21.4')
    scan_folders = Attribute('Scan Folders', min_version='21.4')
    status = Attribute('Status', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.driver_base import DriverBaseAttributes
from pyvenafi.tpp.attributes.monitoring_base import MonitoringBaseAttributes

class X509CertificateBaseAttributes(DriverBaseAttributes, MonitoringBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "X509 Certificate Base"
    adaptable_ca_binary_data_vault_id = Attribute('Adaptable CA:Binary Data Vault ID')
    adaptable_ca_early_password_vault_id = Attribute('Adaptable CA:Early Password Vault ID')
    adaptable_ca_early_pkcs7_vault_id = Attribute('Adaptable CA:Early Pkcs7 Vault ID')
    adaptable_ca_early_private_key_vault_id = Attribute('Adaptable CA:Early Private Key Vault ID')
    adaptable_ca_linked_certificate = Attribute('Adaptable CA:Linked Certificate')
    adaptable_ca_linked_certificate_attribute_name = Attribute('Adaptable CA:Linked Certificate Attribute Name')
    adaptable_ca_linked_certificate_option_enabled = Attribute('Adaptable CA:Linked Certificate Option Enabled')
    adaptable_ca_script_hash_mismatch_error = Attribute('Adaptable CA:Script Hash Mismatch Error')
    adaptable_workflow_approvers = Attribute('Adaptable Workflow Approvers')
    adaptable_workflow_reference_id = Attribute('Adaptable Workflow Reference ID')
    adaptable_workflow_stage = Attribute('Adaptable Workflow Stage')
    address = Attribute('Address')
    algorithm_bits = Attribute('Algorithm Bits')
    algorithm_curve = Attribute('Algorithm Curve')
    algorithm_parameter_set = Attribute('Algorithm Parameter Set')
    algorithm_type = Attribute('Algorithm Type')
    allow_private_key_reuse = Attribute('Allow Private Key Reuse')
    amazon_ca_first_pickup_request = Attribute('Amazon CA:First Pickup Request')
    amazon_ca_timestamp = Attribute('Amazon CA:Timestamp')
    approved_issuer = Attribute('Approved Issuer')
    approver = Attribute('Approver')
    csr_thumbprint = Attribute('CSR Thumbprint')
    csr_vault_id = Attribute('CSR Vault Id')
    certificate_authority = Attribute('Certificate Authority')
    certificate_download_pbes2_algorithm = Attribute('Certificate Download: PBES2 Algorithm')
    certificate_process_validator = Attribute('Certificate Process Validator')
    certificate_vault_id = Attribute('Certificate Vault Id')
    city = Attribute('City')
    comodo_ca_dcv_email = Attribute('Comodo CA:DCV Email')
    comodo_ca_server_type_id = Attribute('Comodo CA:Server Type Id')
    comodo_ccm_ca_pass_phrase = Attribute('Comodo CCM CA:Pass Phrase')
    comodo_ccm_ca_server_type = Attribute('Comodo CCM CA:Server Type')
    consumers = Attribute('Consumers')
    country = Attribute('Country')
    created_by = Attribute('Created By')
    digicert_ca_address = Attribute('DigiCert CA:Address')
    digicert_ca_request_id = Attribute('DigiCert CA:Request Id')
    digicert_ca_server_type = Attribute('DigiCert CA:Server Type')
    digicert_ca_specific_end_date = Attribute('DigiCert CA:Specific End Date')
    digicert_ca_zip = Attribute('DigiCert CA:Zip')
    disable_automatic_renewal = Attribute('Disable Automatic Renewal')
    disable_password_complexity = Attribute('Disable Password Complexity')
    discovered_by_dn = Attribute('Discovered By DN')
    discovered_on = Attribute('Discovered On')
    domain_suffix_whitelist = Attribute('Domain Suffix Whitelist')
    esm_ca_override_default_key_update_policy = Attribute('ESM CA:Override Default Key Update Policy')
    est_reenrollment_in_progress = Attribute('EST ReEnrollment In Progress')
    elliptic_curve = Attribute('Elliptic Curve')
    encryption_driver = Attribute('Encryption Driver')
    enforce_unique_subject = Attribute('Enforce Unique Subject')
    entrust_pki_gateway_early_private_key_vault_id = Attribute('Entrust PKI Gateway:Early Private Key Vault ID')
    entrust_pki_gateway_early_x509_vault_id = Attribute('Entrust PKI Gateway:Early X509 Vault ID')
    entrustnet_ca_additional_emails = Attribute('EntrustNET CA:Additional Emails')
    entrustnet_ca_additional_field_value = Attribute('EntrustNET CA:Additional Field Value')
    entrustnet_ca_email_address = Attribute('EntrustNET CA:Email Address')
    entrustnet_ca_first_name = Attribute('EntrustNET CA:First Name')
    entrustnet_ca_first_pickup_request = Attribute('EntrustNET CA:First Pickup Request')
    entrustnet_ca_last_name = Attribute('EntrustNET CA:Last Name')
    entrustnet_ca_specific_end_date = Attribute('EntrustNET CA:Specific End Date')
    entrustnet_ca_timestamp = Attribute('EntrustNET CA:Timestamp')
    escalation_notice_interval = Attribute('Escalation Notice Interval')
    escalation_notice_start = Attribute('Escalation Notice Start')
    expiration_notice_interval = Attribute('Expiration Notice Interval')
    expiration_notice_start = Attribute('Expiration Notice Start')
    fields = Attribute('Fields')
    generate_keypair_on_application = Attribute('Generate Keypair On Application')
    given_name = Attribute('Given Name')
    globalsign_mssl_ca_email = Attribute('GlobalSign MSSL CA:Email')
    globalsign_mssl_ca_first_name = Attribute('GlobalSign MSSL CA:First Name')
    globalsign_mssl_ca_last_name = Attribute('GlobalSign MSSL CA:Last Name')
    globalsign_mssl_ca_phone = Attribute('GlobalSign MSSL CA:Phone')
    google_cloud_ca_early_x509_chain_vault_id = Attribute('Google Cloud CA:Early X509 Chain Vault ID')
    google_cloud_ca_early_x509_vault_id = Attribute('Google Cloud CA:Early X509 Vault ID')
    grouping_id = Attribute('Grouping Id')
    in_error = Attribute('In Error')
    in_process = Attribute('In Process')
    internet_email_address = Attribute('Internet EMail Address')
    issued_to = Attribute('Issued To')
    jss_certificate_id = Attribute('JSS Certificate ID')
    jss_namespace_dn = Attribute('JSS Namespace DN')
    key_algorithm = Attribute('Key Algorithm')
    key_bit_strength = Attribute('Key Bit Strength')
    key_storage_location = Attribute('Key Storage Location')
    keynectis_sequoia_ca_fields = Attribute('Keynectis Sequoia CA:Fields')
    last_evaluated_on = Attribute('Last Evaluated On')
    last_notification = Attribute('Last Notification')
    last_renewed_by = Attribute('Last Renewed By')
    last_renewed_on = Attribute('Last Renewed On')
    last_validation_state_update = Attribute('Last Validation State Update')
    license_count = Attribute('License Count')
    management_type = Attribute('Management Type')
    manual_approval = Attribute('Manual Approval')
    manual_csr = Attribute('Manual Csr')
    microsoft_ca_request_approved = Attribute('Microsoft CA:Request Approved')
    microsoft_ca_specific_end_date = Attribute('Microsoft CA:Specific End Date')
    network_validation_disabled = Attribute('Network Validation Disabled')
    not_yet_installed_notified_on = Attribute('Not Yet Installed Notified On')
    notified_on = Attribute('Notified On')
    opentrust_pki_ca_fields = Attribute('OpenTrust PKI CA:Fields')
    opentrust_pki_ca_first_pickup_request = Attribute('OpenTrust PKI CA:First Pickup Request')
    opentrust_pki_ca_requester_email = Attribute('OpenTrust PKI CA:Requester Email')
    options = Attribute('Options')
    organization = Attribute('Organization')
    organizational_unit = Attribute('Organizational Unit')
    origin = Attribute('Origin')
    pkcs10_hash_algorithm = Attribute('PKCS10 Hash Algorithm')
    parameter_set = Attribute('Parameter Set')
    placement_id = Attribute('Placement Id')
    postal_code = Attribute('Postal Code')
    private_key_vault_id = Attribute('Private Key Vault Id')
    prohibit_wildcard = Attribute('Prohibit Wildcard')
    prohibited_subject_attributes = Attribute('Prohibited Subject Attributes')
    protection_key = Attribute('Protection Key')
    public_key_vault_id = Attribute('Public Key Vault Id')
    renewal_window = Attribute('Renewal Window')
    reverse_dc_order = Attribute('Reverse DC Order')
    revocation_check_disabled = Attribute('Revocation Check Disabled')
    revocation_check_in_error = Attribute('Revocation Check In Error')
    revocation_check_last_checked = Attribute('Revocation Check Last Checked')
    revocation_check_now = Attribute('Revocation Check Now')
    revocation_check_status = Attribute('Revocation Check Status')
    revocation_original_request = Attribute('Revocation Original Request')
    revocation_request = Attribute('Revocation Request')
    revoke_after = Attribute('Revoke After')
    revoke_prevention_period = Attribute('Revoke Prevention Period')
    scep_transaction_id = Attribute('Scep Transaction Id')
    server_type = Attribute('Server Type')
    signing_request_subject = Attribute('Signing Request Subject')
    specific_end_date = Attribute('Specific End Date')
    stage = Attribute('Stage')
    state = Attribute('State')
    status = Attribute('Status')
    surname = Attribute('Surname')
    telephone = Attribute('Telephone')
    thawte_ca_emails = Attribute('Thawte CA:Emails')
    thawte_ca_enrollment_mode = Attribute('Thawte CA:Enrollment Mode')
    thawte_ca_first_pickup_request = Attribute('Thawte CA:First Pickup Request')
    thawte_ca_timestamp = Attribute('Thawte CA:Timestamp')
    transaction_id = Attribute('Transaction Id')
    trusted_status = Attribute('Trusted Status')
    trustwave_ca_enrollment_mode = Attribute('Trustwave CA:Enrollment Mode')
    trustwave_ca_first_pickup_request = Attribute('Trustwave CA:First Pickup Request')
    trustwave_ca_timestamp = Attribute('Trustwave CA:Timestamp')
    validation_state = Attribute('Validation State')
    validity_period = Attribute('Validity Period')
    want_renewal = Attribute('Want Renewal')
    x509_d = Attribute('X509 D')
    x509_dc = Attribute('X509 DC')
    x509_dnq = Attribute('X509 DNQ')
    x509_e = Attribute('X509 E')
    x509_extension_fields = Attribute('X509 Extension Fields')
    x509_gn = Attribute('X509 GN')
    x509_gq = Attribute('X509 GQ')
    x509_i = Attribute('X509 I')
    x509_p = Attribute('X509 P')
    x509_pa = Attribute('X509 PA')
    x509_pc = Attribute('X509 PC')
    x509_sa = Attribute('X509 SA')
    x509_sn = Attribute('X509 SN')
    x509_sno = Attribute('X509 SNO')
    x509_subject = Attribute('X509 Subject')
    x509_subjectaltname = Attribute('X509 SubjectAltName')
    x509_subjectaltname_dns = Attribute('X509 SubjectAltName DNS')
    x509_subjectaltname_ipaddress = Attribute('X509 SubjectAltName IPAddress')
    x509_subjectaltname_othername_custom = Attribute('X509 SubjectAltName OtherName Custom')
    x509_subjectaltname_othername_upn = Attribute('X509 SubjectAltName OtherName UPN')
    x509_subjectaltname_rfc822 = Attribute('X509 SubjectAltName RFC822')
    x509_subjectaltname_uri = Attribute('X509 SubjectAltName URI')
    x509_t = Attribute('X509 T')
    x509_tn = Attribute('X509 TN')
    x509_ua = Attribute('X509 UA')
    x509_uid = Attribute('X509 UID')
    x509_un = Attribute('X509 UN')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.key_base import KeyBaseAttributes

class KeyPairAttributes(KeyBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Key Pair"
    created_by = Attribute('Created By', min_version='21.4')
    gpg_public_key_vault_id = Attribute('GPG Public Key Vault ID', min_version='21.4')
    key_pair_renewal_flow = Attribute('Key Pair Renewal Flow', min_version='21.4')
    key_usage = Attribute('Key Usage', min_version='22.1')
    original_creation_date = Attribute('Original Creation Date', min_version='21.4')
    owner = Attribute('Owner', min_version='22.1')
    pending_private_key_vault_id = Attribute('Pending Private Key Vault Id', min_version='23.3')
    pending_public_key_vault_id = Attribute('Pending Public Key Vault Id', min_version='23.3')
    public_key_vault_id = Attribute('Public Key Vault Id', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.flow_action_approver_base import FlowActionApproverBaseAttributes

class CodeSigningAdminApprovalActionAttributes(FlowActionApproverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing Admin Approval Action"
    include_code_signing_admin = Attribute('Include Code Signing Admin', min_version='21.4')
    include_master_admin = Attribute('Include Master Admin', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.proxy_base import ProxyBaseAttributes
from pyvenafi.tpp.attributes.top import TopAttributes
from pyvenafi.tpp.attributes.zone_base import ZoneBaseAttributes

class VenafiPlatformAttributes(ProxyBaseAttributes, TopAttributes, ZoneBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Venafi Platform"
    acme_fqdn = Attribute('ACME FQDN', min_version='21.4')
    aws_ec2_role_authorized_identities = Attribute('AWS EC2 Role Authorized Identities', min_version='21.4')
    activation_latency_threshold = Attribute('Activation Latency Threshold', min_version='21.4')
    activation_query_frequency = Attribute('Activation Query Frequency', min_version='21.4')
    activation_work_threshold = Attribute('Activation Work Threshold', min_version='21.4')
    allowed_protocol_version = Attribute('Allowed Protocol Version', min_version='21.4')
    aperture_fqdn = Attribute('Aperture FQDN', min_version='21.4')
    authserver_fqdn = Attribute('AuthServer FQDN', min_version='21.4')
    csc_distribution_fqdn = Attribute('CSC Distribution FQDN', min_version='22.2')
    capabilities_asymmetric_key_generation = Attribute('Capabilities: Asymmetric Key Generation', min_version='21.4')
    certificate_origin = Attribute('Certificate Origin', min_version='21.4')
    certificate_verification = Attribute('Certificate Verification', min_version='21.4')
    certificate_verification_log_warnings = Attribute('Certificate Verification Log Warnings', min_version='21.4')
    check_crl = Attribute('Check CRL', min_version='21.4')
    client_enrollment_require_windows_authentication = Attribute(
        'Client Enrollment Require Windows Authentication',
        min_version='21.4'
    )
    client_id = Attribute('Client ID', min_version='21.4')
    current_upgrade_task = Attribute('Current Upgrade Task', min_version='21.4')
    disable_triggers = Attribute('Disable Triggers', min_version='21.4')
    display_name = Attribute('Display Name', min_version='21.4')
    est_options = Attribute('Est Options', min_version='21.4')
    hsm_fqdn = Attribute('HSM FQDN', min_version='21.4')
    heartbeat_interval = Attribute('Heartbeat Interval', min_version='21.4')
    ignore_identity = Attribute('Ignore Identity', min_version='21.4')
    interval = Attribute('Interval', min_version='21.4')
    keyserver_fqdn = Attribute('KeyServer FQDN', min_version='21.4')
    keystore_cache_check = Attribute('KeyStore Cache Check', min_version='21.4')
    keystore_cache_lifetime = Attribute('KeyStore Cache Lifetime', min_version='21.4')
    keystore_cache_size = Attribute('KeyStore Cache Size', min_version='21.4')
    log_debug = Attribute('Log Debug', min_version='21.4')
    log_server = Attribute('Log Server', min_version='21.4')
    log_target = Attribute('Log Target', min_version='21.4')
    maximum_above_normal_priority_threads = Attribute('Maximum Above Normal Priority Threads', min_version='21.4')
    maximum_below_normal_priority_threads = Attribute('Maximum Below Normal Priority Threads', min_version='21.4')
    maximum_highest_priority_threads = Attribute('Maximum Highest Priority Threads', min_version='21.4')
    maximum_low_priority_threads = Attribute('Maximum Low Priority Threads', min_version='21.4')
    maximum_normal_priority_threads = Attribute('Maximum Normal Priority Threads', min_version='21.4')
    maximum_threads = Attribute('Maximum Threads', min_version='21.4')
    message_bus_self = Attribute('Message Bus Self', min_version='23.3')
    message_bus_self_override = Attribute('Message Bus Self Override', min_version='23.3')
    migration_task = Attribute('Migration Task', min_version='21.4')
    minimum_threads = Attribute('Minimum Threads', min_version='21.4')
    oauth_device_authorization_flow_dn = Attribute('OAuth Device Authorization Flow DN', min_version='22.1')
    operational_certificate_dn = Attribute('Operational Certificate DN', min_version='21.4')
    options = Attribute('Options', min_version='21.4')
    pending_migration_task = Attribute('Pending Migration Task', min_version='21.4')
    portal_fqdn = Attribute('Portal FQDN', min_version='21.4')
    private_key_vault_id = Attribute('Private Key Vault Id', min_version='21.4')
    processing_enabled = Attribute('Processing Enabled', min_version='21.4')
    scep_fqdn = Attribute('SCEP FQDN', min_version='21.4')
    scep_allowed_identities = Attribute('Scep Allowed Identities', min_version='21.4')
    scep_allowed_identities_for_http_authentication = Attribute(
        'Scep Allowed Identities For Http Authentication',
        min_version='21.4'
    )
    scep_certificate_authority = Attribute('Scep Certificate Authority', min_version='21.4')
    scep_challenge_password = Attribute('Scep Challenge Password', min_version='21.4')
    scep_encryption_certificate_authority = Attribute('Scep Encryption Certificate Authority', min_version='21.4')
    scep_encryption_ra_certificate = Attribute('Scep Encryption RA Certificate', min_version='21.4')
    scep_instant_retries_client_user_agent = Attribute('Scep Instant Retries Client User Agent', min_version='21.4')
    scep_intune_application_id = Attribute('Scep Intune Application Id', min_version='21.4')
    scep_intune_application_secret = Attribute('Scep Intune Application Secret', min_version='21.4')
    scep_intune_authentication_authority_resource_url = Attribute(
        'Scep Intune Authentication Authority Resource URL',
        min_version='21.4'
    )
    scep_intune_challenge_password_sanity_check_disabled = Attribute(
        'Scep Intune Challenge Password Sanity Check Disabled',
        min_version='21.4'
    )
    scep_intune_graph_resource_url = Attribute('Scep Intune Graph Resource URL', min_version='21.4')
    scep_intune_http_connection_lease_timeout = Attribute(
        'Scep Intune Http Connection Lease Timeout',
        min_version='21.4'
    )
    scep_intune_http_connection_limit = Attribute('Scep Intune Http Connection Limit', min_version='21.4')
    scep_intune_http_timeout = Attribute('Scep Intune Http Timeout', min_version='21.4')
    scep_intune_ms_graph_resource_url = Attribute('Scep Intune MS Graph Resource URL', min_version='22.2')
    scep_intune_provider_name_and_version = Attribute('Scep Intune Provider Name And Version', min_version='21.4')
    scep_intune_resource_url = Attribute('Scep Intune Resource URL', min_version='21.4')
    scep_intune_tenant_name = Attribute('Scep Intune Tenant Name', min_version='21.4')
    scep_intune_validation_attempts = Attribute('Scep Intune Validation Attempts', min_version='21.4')
    scep_maximum_ndes_challenges = Attribute('Scep Maximum Ndes Challenges', min_version='21.4')
    scep_ndes_challenge_validity = Attribute('Scep Ndes Challenge Validity', min_version='21.4')
    scep_object_container = Attribute('Scep Object Container', min_version='21.4')
    scep_options = Attribute('Scep Options', min_version='21.4')
    scep_ra_certificate = Attribute('Scep RA Certificate', min_version='21.4')
    scep_signing_certificate_authority = Attribute('Scep Signing Certificate Authority', min_version='21.4')
    scep_signing_ra_certificate = Attribute('Scep Signing RA Certificate', min_version='21.4')
    standby_mode = Attribute('Standby Mode', min_version='21.4')
    start_time = Attribute('Start Time', min_version='21.4')
    time_stamping_certificate_dn = Attribute('Time Stamping Certificate DN', min_version='21.4')
    time_stamping_proxy_host_urls = Attribute('Time Stamping Proxy Host URLs', min_version='21.4')
    time_stamping_proxy_options = Attribute('Time Stamping Proxy Options', min_version='21.4')
    timestampserver_fqdn = Attribute('TimeStampServer FQDN', min_version='21.4')
    upgrade_details = Attribute('Upgrade Details', min_version='21.4')
    upgrade_status = Attribute('Upgrade Status', min_version='21.4')
    user_agent_windows_authentication_enabled = Attribute(
        'User Agent Windows Authentication Enabled',
        min_version='21.4'
    )
    vedclient_fqdn = Attribute('VEDClient FQDN', min_version='21.4')
    webadmin_fqdn = Attribute('WebAdmin FQDN', min_version='21.4')
    websdk_fqdn = Attribute('WebSDK FQDN', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_trust_store_base import CertificateTrustStoreBaseAttributes

class PKCS12TrustStoreAttributes(CertificateTrustStoreBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "PKCS#12 Trust Store"
    key_store = Attribute('Key Store', min_version='21.4')
    key_store_credential = Attribute('Key Store Credential', min_version='21.4')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    CloudApiEndpoint,
    CloudApiOutputModel,
    generate_output,
)
from pyvenafi.cloud.api.models import integrations_service
from uuid import UUID

class _integrations_service:
    def __init__(self, api_obj):
        self.v1 = self._v1(api_obj=api_obj)

    class _v1(CloudApiEndpoint):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='v1')
            self.environments = self._environments(api_obj=self._api_obj, url=f'{self._url}/environments')
            self.integrationservices = self._integrationservices(
                api_obj=self._api_obj,
                url=f'{self._url}/integrationservices'
            )
            self.integrationservicesaggregates = self._integrationservicesaggregates(
                api_obj=self._api_obj, url=f'{self._url}/integrationservicesaggregates'
            )

        class _environments(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self, name: str):
                data = {
                    'name': name,
                }

                class Output(CloudApiOutputModel):
                    EnvironmentResponse: integrations_service.EnvironmentResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params=data),
                    rc_mapping={
                        200: 'EnvironmentResponse'
                    }
                )

            class _ID(CloudApiEndpoint):
                def get(self):
                    class Output(CloudApiOutputModel):
                        EnvironmentInformation: integrations_service.EnvironmentInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'EnvironmentInformation'
                        }
                    )

        class _integrationservices(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.scan = self._scan(api_obj=self._api_obj, url=f'{self._url}/scan')

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self, edgeInstanceId: UUID, totalCount: bool):
                data = {
                    'edgeInstanceId': edgeInstanceId,
                    'totalCount'    : totalCount,
                }

                class Output(CloudApiOutputModel):
                    IntegrationServiceDetailsResponse: integrations_service.IntegrationServiceDetailsResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params=data),
                    rc_mapping={
                        200: 'IntegrationServiceDetailsResponse'
                    }
                )

            def post(self, IntegrationServiceCreationRequest: integrations_service.IntegrationServiceCreationRequest):
                data = {**IntegrationServiceCreationRequest.dict()}

                class Output(CloudApiOutputModel):
                    IntegrationServiceInformation: integrations_service.IntegrationServiceInformation

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        201: 'IntegrationServiceInformation'
                    }
                )

            class _ID(CloudApiEndpoint):
                def delete(self, retireCertificates: bool):
                    data = {
                        'retireCertificates': retireCertificates,
                    }

                    class Output(CloudApiOutputModel):
                        pass

                    return generate_output(output_cls=Output, response=self._delete(params=data))

                def get(self):
                    class Output(CloudApiOutputModel):
                        IntegrationServiceInformation: integrations_service.IntegrationServiceInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'IntegrationServiceInformation'
                        }
                    )

                def patch(self, IntegrationServiceUpdateRequest: integrations_service.IntegrationServiceUpdateRequest):
                    data = {**IntegrationServiceUpdateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        IntegrationServiceInformation: integrations_service.IntegrationServiceInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._patch(data=data),
                        rc_mapping={
                            200: 'IntegrationServiceInformation'
                        }
                    )

            class _scan(CloudApiEndpoint):
                def post(
                    self,
                    IntegrationServiceInitiateScanRequest: integrations_service.IntegrationServiceInitiateScanRequest
                ):
                    data = {**IntegrationServiceInitiateScanRequest.dict()}

                    class Output(CloudApiOutputModel):
                        IntegrationServiceInformation: integrations_service.IntegrationServiceInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._post(data=data),
                        rc_mapping={
                            202: 'IntegrationServiceInformation'
                        }
                    )

        class _integrationservicesaggregates(CloudApiEndpoint):
            def get(self):
                class Output(CloudApiOutputModel):
                    IntegrationsServicesAggregatesResponse: integrations_service.IntegrationsServicesAggregatesResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params={}),
                    rc_mapping={
                        200: 'IntegrationsServicesAggregatesResponse'
                    }
                )



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class NetScalerAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "NetScaler"
    certificate_file = Attribute('Certificate File', min_version='21.4')
    chain_cert = Attribute('Chain Cert', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    fips_key = Attribute('Fips Key', min_version='21.4')
    import_only = Attribute('Import Only', min_version='21.4')
    install_path = Attribute('Install Path', min_version='21.4')
    issuer_certificate_name = Attribute('Issuer Certificate Name', min_version='21.4')
    max_filesize = Attribute('Max Filesize', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    partition = Attribute('Partition', min_version='22.2')
    private_key_file = Attribute('Private Key File', min_version='21.4')
    sni_certificate = Attribute('SNI Certificate', min_version='21.4')
    ssl_object_type = Attribute('SSL Object Type', min_version='21.4')
    temp_certificate_label = Attribute('Temp Certificate Label', min_version='21.4')
    virtual_server_name = Attribute('Virtual Server Name', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class PEMAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "PEM"
    certificate_chain_file = Attribute('Certificate Chain File', min_version='21.4')
    certificate_file = Attribute('Certificate File', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    overwrite_existing_chain = Attribute('Overwrite Existing Chain', min_version='21.4')
    pbes2_algorithm = Attribute('PBES2 Algorithm', min_version='21.4')
    private_key_file = Attribute('Private Key File', min_version='21.4')
    private_key_syntax = Attribute('Private Key Syntax')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from datetime import datetime
from typing import (
    Any,
    Dict,
    List,
    Literal,
)
from uuid import UUID

ActivityLogFilterOperator = Literal[
    "EQ",
    "LT",
    "LTE",
    "GT",
    "GTE",
    "IN"
]
AnyValue = Any
Direction = Literal[
    "ASC",
    "DESC"
]
NaryOperator = Literal[
    "AND",
    "OR"
]
UnaryOperator = Literal['NOT']

class ActivityLogCondition(ObjectModel):
    field: str = ApiField(alias='field')
    operator: ActivityLogFilterOperator = ApiField(alias='operator')
    value: str = ApiField(alias='value')
    values: List[str] = ApiField(alias='values', default_factory=list)

class ActivityLogEntriesResponse(ObjectModel):
    activityLogEntries: List[ActivityLogEntryInformation] = ApiField(alias='activityLogEntries', default_factory=list)
    count: int = ApiField(alias='count')

class ActivityLogEntryInformation(ObjectModel):
    activityDate: datetime = ApiField(alias='activityDate')
    activityName: str = ApiField(alias='activityName')
    activityType: str = ApiField(alias='activityType')
    companyId: UUID = ApiField(alias='companyId')
    criticality: int = ApiField(alias='criticality')
    id: UUID = ApiField(alias='id')
    message: str = ApiField(alias='message')
    payload: Dict[str, str] = ApiField(alias='payload', default_factory=dict)

class ActivityLogFilter(ObjectModel):
    expression: BaseActivityLogFilter = ApiField(alias='expression')
    ordering: BaseActivityLogOrdering = ApiField(alias='ordering')
    paging: Page = ApiField(alias='paging')

class ActivityLogFilterOperand(ObjectModel):
    operand: ActivityLogCondition = ApiField(alias='operand')
    operator: UnaryOperator = ApiField(alias='operator')

class ActivityLogFilterOperands(ObjectModel):
    operands: List[BaseActivityLogFilter] = ApiField(alias='operands', default_factory=list)
    operator: NaryOperator = ApiField(alias='operator')

class ActivityLogName(ObjectModel):
    key: str = ApiField(alias='key')
    readableName: str = ApiField(alias='readableName')

class ActivityLogOrder(ObjectModel):
    direction: Direction = ApiField(alias='direction')
    field: str = ApiField(alias='field')

class ActivityLogType(ObjectModel):
    key: str = ApiField(alias='key')
    readableName: str = ApiField(alias='readableName')
    values: List[ActivityLogName] = ApiField(alias='values', default_factory=list)

class BaseActivityLogFilter(ObjectModel):
    ...

class BaseActivityLogOrdering(ObjectModel):
    orders: List[ActivityLogOrder] = ApiField(alias='orders', default_factory=list)

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class ExportedActivityLogEntryInformation(ObjectModel):
    activityDate: datetime = ApiField(alias='activityDate')
    activityName: str = ApiField(alias='activityName')
    activityType: str = ApiField(alias='activityType')
    criticality: int = ApiField(alias='criticality')
    id: UUID = ApiField(alias='id')
    message: str = ApiField(alias='message')
    payload: Dict[str, str] = ApiField(alias='payload', default_factory=dict)

class Page(ObjectModel):
    pageNumber: int = ApiField(alias='pageNumber')
    pageSize: int = ApiField(alias='pageSize')

ActivityLogCondition.update_forward_refs()
ActivityLogEntriesResponse.update_forward_refs()
ActivityLogEntryInformation.update_forward_refs()
ActivityLogFilter.update_forward_refs()
ActivityLogFilterOperand.update_forward_refs()
ActivityLogFilterOperands.update_forward_refs()
ActivityLogName.update_forward_refs()
ActivityLogOrder.update_forward_refs()
ActivityLogType.update_forward_refs()
BaseActivityLogFilter.update_forward_refs()
BaseActivityLogOrdering.update_forward_refs()
ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
ExportedActivityLogEntryInformation.update_forward_refs()
Page.update_forward_refs()



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import (
    identity as ident,
    permissions,
)

class _Permissions(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Permissions')
        self.Object = self._Object(api_obj=self._api_obj, url=f'{self._url}/Object')
        self.Refresh = self._Refresh(api_obj=self._api_obj, url=f'{self._url}/Refresh')

        self.GetPrincipals = self._GetPrincipals(api_obj=self._api_obj, url=f'{self._url}/GetPrincipals')
        self.GetPermissions = self._GetPermissions(api_obj=self._api_obj, url=f'{self._url}/GetPermissions')
        self.GetEffectivePermissions = self._GetEffectivePermissions(
            api_obj=self._api_obj,
            url=f'{self._url}/GetEffectivePermissions'
        )
        self.GrantPermissions = self._GrantPermissions(api_obj=self._api_obj, url=f'{self._url}/GrantPermissions')
        self.RevokePermissions = self._RevokePermissions(api_obj=self._api_obj, url=f'{self._url}/RevokePermissions')
        self.ReplacePermissions = self._ReplacePermissions(api_obj=self._api_obj, url=f'{self._url}/ReplacePermissions')

    class _Object(WebSdkEndpoint):
        def Guid(self, guid):
            return self._Guid(api_obj=self._api_obj, url=f'{self._url}/{guid}')

        class _Guid(WebSdkEndpoint):
            def get(self):
                class Output(WebSdkOutputModel):
                    principals: list[str] = ApiField(default_factory=list)

                return generate_output(output_cls=Output, response=self._get(), root_field='principals')

            def Ptype(self, ptype='Local'):
                return self._Ptype(api_obj=self._api_obj, url=f'{self._url}/{ptype}')

            class _Ptype(WebSdkEndpoint):
                def Pname(self, pname):
                    return self._Pname(api_obj=self._api_obj, url=f'{self._url}/{pname}')

                def Principal(self, uuid: str):
                    return self._Principal(api_obj=self._api_obj, url=f'{self._url}/{uuid}')

                class _Pname(WebSdkEndpoint):
                    def Principal(self, principal: str):
                        return self._Principal(api_obj=self._api_obj, url=f'{self._url}/{principal}')

                    class _Principal(WebSdkEndpoint):
                        def __init__(self, *args, **kwargs):
                            super().__init__(*args, **kwargs)
                            self.Effective = self._Effective(api_obj=self._api_obj, url=f'{self._url}/Effective')

                        def delete(self):
                            return generate_output(output_cls=WebSdkOutputModel, response=self._delete())

                        def get(self):
                            class Output(WebSdkOutputModel):
                                explicit_permissions: permissions.Permissions = ApiField(alias='ExplicitPermissions')
                                implicit_permissions: permissions.Permissions = ApiField(alias='ImplicitPermissions')

                            return generate_output(output_cls=Output, response=self._get())

                        def post(
                            self,
                            is_associate_allowed: bool = None,
                            is_create_allowed: bool = None,
                            is_delete_allowed: bool = None,
                            is_manage_permissions_allowed: bool = None,
                            is_policy_write_allowed: bool = None,
                            is_private_key_read_allowed: bool = None,
                            is_private_key_write_allowed: bool = None,
                            is_read_allowed: bool = None,
                            is_rename_allowed: bool = None,
                            is_revoke_allowed: bool = None,
                            is_view_allowed: bool = None,
                            is_write_allowed: bool = None
                        ):
                            body = {
                                'IsAssociateAllowed'        : is_associate_allowed,
                                'IsCreateAllowed'           : is_create_allowed,
                                'IsDeleteAllowed'           : is_delete_allowed,
                                'IsManagePermissionsAllowed': is_manage_permissions_allowed,
                                'IsPolicyWriteAllowed'      : is_policy_write_allowed,
                                'IsPrivateKeyReadAllowed'   : is_private_key_read_allowed,
                                'IsPrivateKeyWriteAllowed'  : is_private_key_write_allowed,
                                'IsReadAllowed'             : is_read_allowed,
                                'IsRenameAllowed'           : is_rename_allowed,
                                'IsRevokeAllowed'           : is_revoke_allowed,
                                'IsViewAllowed'             : is_view_allowed,
                                'IsWriteAllowed'            : is_write_allowed
                            }

                            return generate_output(output_cls=WebSdkOutputModel, response=self._post(data=body))

                        def put(
                            self,
                            is_associate_allowed: bool = None,
                            is_create_allowed: bool = None,
                            is_delete_allowed: bool = None,
                            is_manage_permissions_allowed: bool = None,
                            is_policy_write_allowed: bool = None,
                            is_private_key_read_allowed: bool = None,
                            is_private_key_write_allowed: bool = None,
                            is_read_allowed: bool = None,
                            is_rename_allowed: bool = None,
                            is_revoke_allowed: bool = None,
                            is_view_allowed: bool = None,
                            is_write_allowed: bool = None
                        ):
                            body = {
                                'IsAssociateAllowed'        : is_associate_allowed,
                                'IsCreateAllowed'           : is_create_allowed,
                                'IsDeleteAllowed'           : is_delete_allowed,
                                'IsManagePermissionsAllowed': is_manage_permissions_allowed,
                                'IsPolicyWriteAllowed'      : is_policy_write_allowed,
                                'IsPrivateKeyReadAllowed'   : is_private_key_read_allowed,
                                'IsPrivateKeyWriteAllowed'  : is_private_key_write_allowed,
                                'IsReadAllowed'             : is_read_allowed,
                                'IsRenameAllowed'           : is_rename_allowed,
                                'IsRevokeAllowed'           : is_revoke_allowed,
                                'IsViewAllowed'             : is_view_allowed,
                                'IsWriteAllowed'            : is_write_allowed
                            }

                            return generate_output(output_cls=WebSdkOutputModel, response=self._put(data=body))

                        class _Effective(WebSdkEndpoint):
                            def get(self):
                                class Output(WebSdkOutputModel):
                                    effective_permissions: permissions.Permissions = ApiField(
                                        alias='EffectivePermissions'
                                    )

                                return generate_output(output_cls=Output, response=self._get())

                class _Principal(WebSdkEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)
                        self.Effective = self._Effective(api_obj=self._api_obj, url=f'{self._url}/Effective')

                    def delete(self):
                        return generate_output(output_cls=WebSdkOutputModel, response=self._delete())

                    def get(self):
                        class Output(WebSdkOutputModel):
                            explicit_permissions: permissions.Permissions = ApiField(alias='ExplicitPermissions')
                            implicit_permissions: permissions.Permissions = ApiField(alias='ImplicitPermissions')

                        return generate_output(output_cls=Output, response=self._get())

                    def post(
                        self,
                        is_associate_allowed: bool = None,
                        is_create_allowed: bool = None,
                        is_delete_allowed: bool = None,
                        is_manage_permissions_allowed: bool = None,
                        is_policy_write_allowed: bool = None,
                        is_private_key_read_allowed: bool = None,
                        is_private_key_write_allowed: bool = None,
                        is_read_allowed: bool = None,
                        is_rename_allowed: bool = None,
                        is_revoke_allowed: bool = None,
                        is_view_allowed: bool = None,
                        is_write_allowed: bool = None
                    ):
                        body = {
                            'IsAssociateAllowed'        : is_associate_allowed,
                            'IsCreateAllowed'           : is_create_allowed,
                            'IsDeleteAllowed'           : is_delete_allowed,
                            'IsManagePermissionsAllowed': is_manage_permissions_allowed,
                            'IsPolicyWriteAllowed'      : is_policy_write_allowed,
                            'IsPrivateKeyReadAllowed'   : is_private_key_read_allowed,
                            'IsPrivateKeyWriteAllowed'  : is_private_key_write_allowed,
                            'IsReadAllowed'             : is_read_allowed,
                            'IsRenameAllowed'           : is_rename_allowed,
                            'IsRevokeAllowed'           : is_revoke_allowed,
                            'IsViewAllowed'             : is_view_allowed,
                            'IsWriteAllowed'            : is_write_allowed
                        }

                        return generate_output(output_cls=WebSdkOutputModel, response=self._post(data=body))

                    def put(
                        self,
                        is_associate_allowed: bool = None,
                        is_create_allowed: bool = None,
                        is_delete_allowed: bool = None,
                        is_manage_permissions_allowed: bool = None,
                        is_policy_write_allowed: bool = None,
                        is_private_key_read_allowed: bool = None,
                        is_private_key_write_allowed: bool = None,
                        is_read_allowed: bool = None,
                        is_rename_allowed: bool = None,
                        is_revoke_allowed: bool = None,
                        is_view_allowed: bool = None,
                        is_write_allowed: bool = None
                    ):
                        body = {
                            'IsAssociateAllowed'        : is_associate_allowed,
                            'IsCreateAllowed'           : is_create_allowed,
                            'IsDeleteAllowed'           : is_delete_allowed,
                            'IsManagePermissionsAllowed': is_manage_permissions_allowed,
                            'IsPolicyWriteAllowed'      : is_policy_write_allowed,
                            'IsPrivateKeyReadAllowed'   : is_private_key_read_allowed,
                            'IsPrivateKeyWriteAllowed'  : is_private_key_write_allowed,
                            'IsReadAllowed'             : is_read_allowed,
                            'IsRenameAllowed'           : is_rename_allowed,
                            'IsRevokeAllowed'           : is_revoke_allowed,
                            'IsViewAllowed'             : is_view_allowed,
                            'IsWriteAllowed'            : is_write_allowed
                        }

                        return generate_output(output_cls=WebSdkOutputModel, response=self._put(data=body))

                    class _Effective(WebSdkEndpoint):
                        def get(self):
                            class Output(WebSdkOutputModel):
                                effective_permissions: permissions.Permissions = ApiField(alias='EffectivePermissions')

                            return generate_output(output_cls=Output, response=self._get())

    class _Refresh(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')

            return generate_output(output_cls=Output, response=self._get())

    class _GetPrincipals(WebSdkEndpoint):
        def post(self, object_dn: str = None, object_guid: str = None):
            body = {
                "ObjectDN"  : object_dn,
                "ObjectGuid": object_guid,
            }

            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')
                principals: list[ident.Identity] = ApiField(alias='Principals')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetPermissions(WebSdkEndpoint):
        def post(
            self,
            principal: str = None,
            object_dn: str = None,
            object_guid: str = None,
        ):
            body = {
                "Principal" : principal,
                "ObjectDN"  : object_dn,
                "ObjectGuid": object_guid,
            }

            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')
                permissions: permissions.Permissions = ApiField(alias='Permissions')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetEffectivePermissions(WebSdkEndpoint):
        def post(
            self,
            principal: str = None,
            object_dn: str = None,
            object_guid: str = None,
        ):
            body = {
                "Principal" : principal,
                "ObjectDN"  : object_dn,
                "ObjectGuid": object_guid,
            }

            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')
                permissions: permissions.Permissions = ApiField(alias='Permissions')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GrantPermissions(WebSdkEndpoint):
        def post(
            self,
            principal: str = None,
            object_dn: str = None,
            object_guid: str = None,
        ):
            body = {
                "Principal" : principal,
                "ObjectDN"  : object_dn,
                "ObjectGuid": object_guid,
            }

            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RevokePermissions(WebSdkEndpoint):
        def post(
            self,
            principal: str = None,
            object_dn: str = None,
            object_guid: str = None,
        ):
            body = {
                "Principal" : principal,
                "ObjectDN"  : object_dn,
                "ObjectGuid": object_guid,
            }

            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ReplacePermissions(WebSdkEndpoint):
        def post(
            self,
            principal: str = None,
            object_dn: str = None,
            object_guid: str = None,
        ):
            body = {
                "Principal" : principal,
                "ObjectDN"  : object_dn,
                "ObjectGuid": object_guid,
            }

            class Output(WebSdkOutputModel):
                result: int = ApiField(alias='Result')

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.code_signing_environment_template_base import CodeSigningEnvironmentTemplateBaseAttributes

class CodeSigningKeyPairEnvironmentTemplateAttributes(
    CodeSigningEnvironmentTemplateBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "Code Signing Key Pair Environment Template"
    key_algorithm = Attribute('Key Algorithm', min_version='21.4')
    key_container_dn = Attribute('Key Container DN', min_version='21.4')
    key_storage_location = Attribute('Key Storage Location', min_version='21.4')
    max_uses = Attribute('Max Uses', min_version='21.4')
    validity_period = Attribute('Validity Period', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.client_work_base import ClientWorkBaseAttributes

class ClientAgentAutomaticUpgradeWorkAttributes(ClientWorkBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Client Agent Automatic Upgrade Work"
    days_of_month = Attribute('Days Of Month', min_version='21.4')
    days_of_week = Attribute('Days Of Week', min_version='21.4')
    force_agent_upgrade = Attribute('Force Agent Upgrade', min_version='21.4')
    interval = Attribute('Interval', min_version='21.4')
    schedule_type = Attribute('Schedule Type', min_version='21.4')
    start_time = Attribute('Start Time', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.report_base import ReportBaseAttributes
from pyvenafi.tpp.attributes.report_filter_base import ReportFilterBaseAttributes

class ReportDemographicsWildcardAttributes(ReportBaseAttributes, ReportFilterBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Report:Demographics Wildcard"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class TealeafSunimpAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "TealeafSunimp"
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    key_store_credential = Attribute('Key Store Credential', min_version='21.4')
    key_store_name = Attribute('Key Store Name', min_version='21.4')
    key_store_validation_disabled = Attribute('Key Store Validation Disabled', min_version='21.4')
    sunimp_utility_path = Attribute('Sunimp Utility Path', min_version='21.4')
    tealeaf_utility_path = Attribute('Tealeaf Utility Path', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import config_schema

class _ConfigSchema(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/ConfigSchema')
        self.Attributes = self._Attributes(api_obj=api_obj, url=f'{self._url}/Attributes')
        self.Class = self._Class(api_obj=api_obj, url=f'{self._url}/Class')

    class _Attributes(WebSdkEndpoint):
        def post(self):
            class Output(WebSdkOutputModel):
                attribute_definitions: list[config_schema.AttributeDefinition] = ApiField(
                    default_factory=list, alias='AttributeDefinitions'
                )
                result: config_schema.Result = ApiField(
                    alias='Result',
                    converter=lambda x: config_schema.Result(code=x)
                )

            return generate_output(response=self._post(data={}), output_cls=Output)

    class _Class(WebSdkEndpoint):
        def post(self, class_name: str):
            body = {
                "Class": class_name
            }

            class Output(WebSdkOutputModel):
                class_definition: config_schema.ClassDefinition = ApiField(alias='ClassDefinition')
                result: config_schema.Result = ApiField(
                    alias='Result',
                    converter=lambda x: config_schema.Result(code=x)
                )

            return generate_output(response=self._post(data=body), output_cls=Output)



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes

class SelfSignedCAAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Self Signed CA"
    algorithm = Attribute('Algorithm', min_version='21.4')
    enhanced_key_usage = Attribute('Enhanced Key Usage', min_version='21.4')
    key_usage = Attribute('Key Usage', min_version='21.4')
